#include "Ayudame.h"

#include <cstdio>
#include <cstring>
#include <pthread.h>
#include <map>
#include <set>
 
// forward function declarations
int AYU_read(int c, void *buf, size_t buf_size);
void *AYU_receiveThreadFunction(void *c_);
int AYU_initControl (intptr_t c);
void AYU_pause(ayu_event_t event = AYU_EVENT_NULL, int taskId = 0);

pthread_t AYU_recvThread;	// thread handle to receive thread

// mutex for variable AYU_breakpoint and for pause and step flags
pthread_mutex_t AYU_breakpointMutex = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t AYU_pauseMutex = PTHREAD_MUTEX_INITIALIZER;

std::set < int >AYU_blockedTasks;	// task Ids of blocked tasks.
std::map < int, int >AYU_taskPriority;	// map of task ids and priority level

int AYU_breakpoint = 0;			// breakpoint flag
int AYU_pauseOn_anyEvent = 0;	// pause on any event flag
int AYU_pauseOn_PRERUNTASK = 0;	// pause on AYU_PRERUNTASK flag
int AYU_pauseOn_REMOVETASK = 0;	// pause on AYU_REMOVETASK flag
int AYU_pauseOn_WAITON = 0;		// pause on AYU_WAITON flag
int AYU_pauseOn_BARRIER = 0;	// pause on AYU_BARRIER flag
int AYU_pauseOn_task = -1;		// pause on task. Value is task id, -1 is off
int AYU_pauseOn_function = -1;	// pause on function. Value is task id, -1 is off
int AYU_task_blocks = 0;		// flag whether a AYU_pauseOn_task is active
int AYU_function_blocks = 0;	// flag whether a AYU_pauseOn_function is active
int AYU_step = 0;				// step ("un-pause" current event) flag

/****f* AYUDAME/AYU_getBreakpoint
 *  NAME
 *    AYU_getBreakpoint
 *    - return AYU_breakpoint
 *  SYNOPSIS
 *    breakpoint = AYU_getBreakpoint()
 *  DESCRIPTION
 *    Locks the global variable AYU_breakpoint, returns its value
 *    and unlocks it
 *  PARAMETERS
 *    void
 *  RETURN VALUE
 *    1 if AYU_breakpoint is set
 *    0 if AYU_breakpoint is not set
 *  SOURCE
 */
int AYU_getBreakpoint(void)
{
	int breakpoint;
	pthread_mutex_lock(&AYU_breakpointMutex);
	breakpoint = AYU_breakpoint;
	pthread_mutex_unlock(&AYU_breakpointMutex);
	return breakpoint;
}
/******/


/****f* AYUDAME/AYU_initControl
 *  NAME
 *    AYU_initControl
 *    - initialise receive thread for server side communication
 *  SYNOPSIS
 *    AYU_initControl(c)
 *  DESCRIPTION
 *    Initialises the receive thread for server side communication
 *    for the AYUDAME tool
 *  PARAMETERS
 *    intptr_t c                // client handle
 *  RETURN VALUE
 *    void
 *  SOURCE
 */

int AYU_initControl (intptr_t c)
{
	pthread_create(&AYU_recvThread, NULL, AYU_receiveThreadFunction,
		       (void *)c);

    /*
     * char *envVar = getenv(NUM_CPUS_ENVVAR); if (envVar != NULL) {
     * AYU_NUM_CPUS = atoi(envVar); #if AYU_VERBOSE > 0 std::cout<<
     * "AYUDAME: found " << NUM_CPUS_ENVVAR << " = " << AYU_NUM_CPUS
     * << std::endl; #endif }
     */

	return 0;
}

/******/

/****f* AYUDAME/AYU_isBlocked
 *  NAME
 *    AYU_isBlocked
 *    - return true if task is blocked
 *  SYNOPSIS
 *    if(AYU_isBlocked(taskId)) { do_something(); }
 *  DESCRIPTION
 *    Locks the global variable AYU_blockedTask using the
 *    breakpoint mutex lock, compares to it
 *    taskId given as parameter and returns result
 *  PARAMETERS
 *    * const int taskId
 *  RETURN VALUE
 *    true if task is blocked
 *    false if task is not blocked
 *  SOURCE
 */
bool AYU_isBlocked(const int taskId)
{
	bool isBlocked;
	pthread_mutex_lock(&AYU_breakpointMutex);
	isBlocked = (AYU_blockedTasks.count(taskId) > 0);
	pthread_mutex_unlock(&AYU_breakpointMutex);
	return isBlocked;
}
/******/

/****f* AYUDAME/AYU_pause
 *  NAME
 *    AYU_pause
 *    - pauses execution while AYU_pauseOn* switches are set
 *  SYNOPSIS
 *    AYU_pause(event)
 *  DESCRIPTION
 *    Pauses the execution when AYU_pauseOn* switches are 1 by
 *    entering a loop
 *  PARAMETERS
 *    * ayu_event_t event
 *    * int taskId
 *  RETURN VALUE
 *    void
 *  SOURCE
 */
void AYU_pause(ayu_event_t event, int taskId)
{
	int pause = -1;		// local copy of AYU_pauseOn_*
	
	// check for AYU_pauseOn_anyEvent
	pthread_mutex_lock(&AYU_pauseMutex);
	pause = AYU_pauseOn_anyEvent;
	pthread_mutex_unlock(&AYU_pauseMutex);
	if (pause == 1) {
		while (1) {
			pthread_mutex_lock(&AYU_pauseMutex);
			if (AYU_step > 0) {
				AYU_step--;
				pthread_mutex_unlock(&AYU_pauseMutex);
				return;
			}
			pthread_mutex_unlock(&AYU_pauseMutex);
			usleep(1.e3);
		}
	}
	// check for AYU_task_blocks
	pthread_mutex_lock(&AYU_pauseMutex);
	pause = AYU_task_blocks;
	pthread_mutex_unlock(&AYU_pauseMutex);
	if (pause == 1) {
		while (1) {
			pthread_mutex_lock(&AYU_pauseMutex);
			if (AYU_task_blocks == 0) {
				pthread_mutex_unlock(&AYU_pauseMutex);
				return;
			}
			pthread_mutex_unlock(&AYU_pauseMutex);
			usleep(1.e3);
		}
	}
	// check for other events
	switch (event) {
	case AYU_INIT:
		while (1) {
			pthread_mutex_lock(&AYU_pauseMutex);
	       		if (AYU_step > 0) {
				AYU_step--;
				pthread_mutex_unlock(&AYU_pauseMutex);
				return;
			}
			pthread_mutex_unlock(&AYU_pauseMutex);
			usleep(1.e3);
		}
		break;
	case AYU_PRERUNTASK:
		pthread_mutex_lock(&AYU_pauseMutex);
		pause = AYU_pauseOn_PRERUNTASK;
		if (taskId == AYU_pauseOn_task) {
			//css_set_num_threads(1);
			AYU_task_blocks = 1;
			pause = 1;
		}
		pthread_mutex_unlock(&AYU_pauseMutex);
		if (pause == 1) {
			while (1) {
				pthread_mutex_lock(&AYU_pauseMutex);
				if (AYU_step > 0) {
					AYU_step--;
					AYU_task_blocks = 0;
					/*if (taskId == AYU_pauseOn_task){
					   css_set_num_threads(AYU_NUM_CPUS);
					   } */
					pthread_mutex_unlock(&AYU_pauseMutex);
					return;
				}
				pthread_mutex_unlock(&AYU_pauseMutex);
				usleep(1.e3);
			}
		}
		break;
	case AYU_REMOVETASK:
		pthread_mutex_lock(&AYU_pauseMutex);
		pause = AYU_pauseOn_REMOVETASK;
		pthread_mutex_unlock(&AYU_pauseMutex);
		if (pause == 1) {
			while (1) {
				pthread_mutex_lock(&AYU_pauseMutex);
				if (AYU_step > 0) {
					AYU_step--;
					pthread_mutex_unlock(&AYU_pauseMutex);
					return;
				}
				pthread_mutex_unlock(&AYU_pauseMutex);
				usleep(1.e3);
			}
		}
		break;
	case AYU_WAITON:
		pthread_mutex_lock(&AYU_pauseMutex);
		pause = AYU_pauseOn_WAITON;
		pthread_mutex_unlock(&AYU_pauseMutex);
		if (pause == 1) {
			while (1) {
				pthread_mutex_lock(&AYU_pauseMutex);
				if (AYU_step > 0) {
					AYU_step--;
					pthread_mutex_unlock(&AYU_pauseMutex);
					return;
				}
				pthread_mutex_unlock(&AYU_pauseMutex);
				usleep(1.e3);
			}
		}
		break;
	case AYU_BARRIER:
		pthread_mutex_lock(&AYU_pauseMutex);
		pause = AYU_pauseOn_BARRIER;
		pthread_mutex_unlock(&AYU_pauseMutex);
		if (pause == 1) {
			while (1) {
				pthread_mutex_lock(&AYU_pauseMutex);
				if (AYU_step > 0) {
					AYU_step--;
					pthread_mutex_unlock(&AYU_pauseMutex);
					return;
				}
				pthread_mutex_unlock(&AYU_pauseMutex);
				usleep(1.e3);
			}
		}
		break;
	default:
	  fprintf(stderr,"AYUDAME: wrong event for AYU_pause! event: %d\n",event);
	}

}
/******/

/****f* AYUDAME/AYU_read
 *  NAME
 *    AYU_read
 *    - read data from a socket connection
 *  SYNOPSIS
 *    n = AYU_read(c,buf,buf_size)
 *  DESCRIPTION
 *    reads data from a socket connection accessed by a handle
 *  PARAMETERS
 *    int c                // client handle
 *    void *buf            // pointer to buffer
 *    size_t buf_size      // size of data chunk in bytes
 *  RETURN VALUE
 *    integer number of recieved bytes, -1 on failure
 *  SOURCE
 */
int AYU_read(int c, void *buf, size_t buf_size)
{
	int n = -1;

	for (unsigned int i = 0; i < buf_size; i += n) {
		n = read(c, (uint8_t *) buf + i, buf_size - i);
		if (n < 0) {
			perror("ERROR reading from client socket");
			return -1;
		}
	}

#if AYU_WRITETOSTDOUT
	char outbuf[128];
	char tmpbuf[128];
	memset (outbuf, '\0', 128);
	memset (tmpbuf, '\0', 128);
	sprintf(outbuf,"AYUDAME_RECV:");
  	for (unsigned int i = 0; i < buf_size / sizeof(uint64_t); ++i) {
	  sprintf(tmpbuf," %ld",be64toh (((uint64_t *) buf)[i]));
	  strcat(outbuf,tmpbuf);
	}
	printf("%s\n",outbuf);
#endif

	return n;
}
/******/

/****f* AYUDAME/AYU_receiveThreadFunction
 *  NAME
 *    AYU_receiveThreadFunction
 *    - receive and react to messages
 *  SYNOPSIS
 *    pthread_create(&thread, NULL, AYU_receiveThreadFunction, client);
 *  DESCRIPTION
 *    Implements the infinite loop to receive messages from client and react.
 *  PARAMETERS
 *    void *c_                // client handle
 *  RETURN VALUE
 *    void
 *  SOURCE
 */
void *AYU_receiveThreadFunction(void *c_)
{
	// initialise variables ----------------------

	intptr_t c = (intptr_t) c_;	// client handle
	int n = -1;		// return of read and write
	int64_t buf[AYU_buf_size];	// send and recieve buffer
	//	int recv_taskId = -1;	// task Id of received message
	ayu_request_t request = AYU_REQUEST_NULL;	// variable to store request from client
	ayu_event_t event = AYU_EVENT_NULL;	// variable to store event from client
	int value = -1;		// received value
	int taskId = -1;	// task Id variable
	std::pair<std::map<int,int>::iterator,bool> ret;
	// -------------------------------------------

	while (1) {

		// read from client --------------------------

		n = AYU_read(c, (void *)buf, sizeof(buf));
		if (n < 0)
			return NULL;
		// -------------------------------------------

		// respond to request -----------------------

		//	recv_taskId = be64toh(buf[0]);
		request = (ayu_request_t) be64toh(buf[1]);

		switch (request) {

		case AYU_NOREQUEST:
		        break;

		case AYU_BREAKPOINT:
			value = be64toh(buf[3]);
			pthread_mutex_lock(&AYU_breakpointMutex);
			AYU_breakpoint = value;
			pthread_mutex_unlock(&AYU_breakpointMutex);
			break;

		case AYU_PAUSEONEVENT:
			event = (ayu_event_t) be64toh(buf[2]);
			value = be64toh(buf[3]);

			switch (event) {
			case AYU_EVENT_NULL:
				pthread_mutex_lock(&AYU_pauseMutex);
				AYU_pauseOn_anyEvent = value;
				pthread_mutex_unlock(&AYU_pauseMutex);
				break;
			case AYU_PRERUNTASK:
				pthread_mutex_lock(&AYU_pauseMutex);
				AYU_pauseOn_PRERUNTASK = value;
				pthread_mutex_unlock(&AYU_pauseMutex);
				break;
			case AYU_REMOVETASK:
				pthread_mutex_lock(&AYU_pauseMutex);
				AYU_pauseOn_REMOVETASK = value;
				pthread_mutex_unlock(&AYU_pauseMutex);
				break;
			case AYU_WAITON:
				pthread_mutex_lock(&AYU_pauseMutex);
				AYU_pauseOn_WAITON = value;
				pthread_mutex_unlock(&AYU_pauseMutex);
				break;
			case AYU_BARRIER:
				pthread_mutex_lock(&AYU_pauseMutex);
				AYU_pauseOn_BARRIER = value;
				pthread_mutex_unlock(&AYU_pauseMutex);
				break;
			default:
				break;
			}
			break;

		case AYU_STEP:
			value = be64toh(buf[2]);
			pthread_mutex_lock(&AYU_pauseMutex);
			AYU_step = value;
			pthread_mutex_unlock(&AYU_pauseMutex);
			break;

		case AYU_PAUSEONTASK:
			if (be64toh(buf[3]) == 0) {
				pthread_mutex_lock(&AYU_pauseMutex);
				AYU_pauseOn_task = -1;
				pthread_mutex_unlock(&AYU_pauseMutex);
			} else if (be64toh(buf[3]) == 1) {
				pthread_mutex_lock(&AYU_pauseMutex);
				AYU_pauseOn_task = be64toh(buf[2]);
				pthread_mutex_unlock(&AYU_pauseMutex);
			}

			break;

		case AYU_PAUSEONFUNCTION:
			break;

		case AYU_BLOCKTASK:
			taskId = be64toh(buf[2]);
			value = be64toh(buf[3]);
			pthread_mutex_lock(&AYU_breakpointMutex);
			if (value == 1)
				AYU_blockedTasks.insert(taskId);
			else
				AYU_blockedTasks.erase(taskId);
			pthread_mutex_unlock(&AYU_breakpointMutex);

			break;

		case AYU_PRIORITISETASK:
			taskId = be64toh(buf[2]);
			value = be64toh(buf[3]);
			pthread_mutex_lock(&AYU_breakpointMutex);
			ret = AYU_taskPriority.insert(std::make_pair(taskId,value));
			if (ret.second == false) { // taskId already existed
			  ret.first->second = value; // set priority level to value
			}
			pthread_mutex_unlock(&AYU_breakpointMutex);

			break;

		case AYU_SETNUMTHREADS:
			value = be64toh(buf[2]);
			if((unsigned)value > AYU_max_threads){
			  fprintf(stderr,"AYUDAME ERROR: setting number of threads greater than %ld is not possible!\n",AYU_max_threads);
			}
			if((unsigned)value <= AYU_max_threads){
			  // TODO:
			  if (AYU_setNumThreads) { AYU_setNumThreads(value); }
			  else printf("AYU_setNumThreads does not exist.\n");
			}
			break;

		default:
		        fprintf(stderr,"AYUDAME: ERROR: received unknown request code %d\n", request);
			break;
		}
		// --------------------

		// -------------------------------------------

	}

	return NULL;
}
/******/

/****f* AYUDAME/AYU_getPriorityLevel
 *  NAME
 *    AYU_getPriorityLevel
 *    - returns priority of task
 *  SYNOPSIS
 *    AYU_getPriorityLevel(taskId)
 *  DESCRIPTION
 *    returns the priority of a task identified by its id.
 *    If the priority has not been changed in Ayudame it
 *    returns -1.
 *  PARAMETERS
 *    int taskId
 *  RETURN VALUE
 *    -1: if priority has not changed
 *    0:  low (no) priority
 *    positive integer: priority level
 *  SOURCE
 */
int AYU_getPriorityLevel(int taskId)
{
  pthread_mutex_lock(&AYU_breakpointMutex);
  auto it = AYU_taskPriority.find(taskId);
  auto end = AYU_taskPriority.end();
  pthread_mutex_unlock(&AYU_breakpointMutex);

  if ( it == end ){
    return -1;
  }
  else {
    return (*it).second;
  }
  
}
/******/
