/****h* *Ss-Tools/AYUDAME
 *  NAME
 *    AYUDAME
 *    - package to communicate between *Ss runtime and external tools
 *  DESCRIPTION
 *    AYUDAME provides a callback function AYU_event() which is called
 *    from the *Ss runtime. It processes the information given to it
 *    via the function parameters and passes it to clients via sockets.
 *
 *    |html <img src="AYUDAME_scheme.png" alt="AYUDAME_scheme.png missing"/>
 *    |latex \includegraphics{AYUDAME_scheme.png}
 *
 *    AYU_event() is declared with __attribute__ ((weak)) in Ayudame.h
 *    which has to be included in the runtime.
 *    The events and requests are declared in Ayudame_types.h
 *  USAGE
 *    To compile the SMPSs runtime with AYUDAME support, configure SMPSs
 *    with --enable-ayudame, to compile without AYUDAME support, use
 *    --disable-ayudame. Possibly it is necessary to do a autoreconf to
 *    generate the right configure file.
 *    Without reconfiguring, you can add/edit the line in master/config.h to
 *      #define USE_AYUDAME 1 // 1 for yes, 0 for no
 *    Finally you have to (re)compile SMPSs with
 *      make clean && make && make install
 *
 *    Additionally one has to compile the libayudame.so. To do so go to the
 *    Tools/ayudame directory and perform a simple
 *      make
 *    Run you application (which you compiled with smpss-cc) this way to
 *    use the AYUDAME tool:
 *      export AYU_PORT=<port>
 *      LD_PRELOAD=/path/to/libayudame.so ./smpss_app
 *    The execution will be paused until a client connects to the AYUDAME
 *    library. Alternatively you can run temanejo.py and start the application
 *    from there. For details refer to the TEMANEJO documentation.
 *  SIDE EFFECTS
 *    Some ouput is printed to stdout. For quiet operation reroute
 *    to file or /dev/null or define AYU_VERBOSE = -1
 *  TODO
 *    * possibility for clients to connect and disconnect whenever
 *  AUTHOR
 *    Steffen Brinkmann, HLRS <brinkmann@hlrs.de>
 *  COPYRIGHT
 *    (C) HLRS, University of Stuttgart
 *    AYUDAME is published under the terms of the BSD license.
 ******/

#include "Ayudame.h"

#include <dlfcn.h>
#include <execinfo.h>

//#include <cxxabi.h>

//#define UNW_LOCAL_ONLY
//#include <libunwind.h>

#include <pthread.h>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include <iostream>
#include <string>

#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>

// mutex for variables AYU_client and for messages
pthread_mutex_t AYU_clientMutex = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t AYU_msgMutex = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t AYU_countMutex = PTHREAD_MUTEX_INITIALIZER;

int AYU_client = -1;		// client handle

unsigned long AYU_n_threads;       // current number of threads
unsigned long AYU_max_threads;     // maximum number of threads

// forward declarations
/*
  extern "C"{
  int getFileAndLine (unw_word_t addr, char *file, size_t flen, int *line);
  void show_backtrace (void);
  } // extern "C"
*/

int AYU_initSocket (void);
int AYU_write (int c, void *buf, size_t buf_size, bool isString=false);
extern "C"
{
  uint64_t AYU_rdtsc (void);
}


int AYU_initControl (intptr_t c);
void AYU_pause (ayu_event_t event = AYU_EVENT_NULL, int taskId = 0);

/****f* AYUDAME/AYU_event
 *  NAME
 *    AYU_event
 *    - callback for the *Ss runtime
 *  SYNOPSIS
 *    AYU_event(event, taskId, pointer)
 *  DESCRIPTION
 *    this function is called from the *Ss runtime to communicate
 *    with the AYUDAME package. Events of type ayu_event_t, the task ID
 *    of the calling task and a pointer to arbitrary data can be sent.
 *    AYU_event will connect to a client via tcp sockets and pass on
 *    the information.
 *  PARAMETERS
 *    ayu_event_t event      // type of event
 *    const int64_t taskId   // taskId from which the event is sent
 *    void *p                // pointer to further data
 *  RETURN VALUE
 *    void
 *  SOURCE
 ma */
void
AYU_event (ayu_event_t event, const int64_t taskId, void *p)
{
  //int i = 0;			// iteration variable
  int n = -1;			// return of read and write
  int c = -1;			// client handle
  const size_t buf_size = 8;	// size of message buffer
  int64_t buf[buf_size];	// send and recieve buffer
  const size_t string_buf_size = 128;	// size of string buffer
  char string_buf[string_buf_size];	// string buffer 
  //static int preselect_count = 0;	// preselect task event count
  static int regfunc_count = 0;	// register function count
  static ayu_runtime_t runtimeId = AYU_RT_UNKNOWN;
  static int ayu_event_count = 0;	// preselect task event count

  //  return;
  pthread_mutex_lock (&AYU_countMutex);
  ++ayu_event_count;
  pthread_mutex_unlock (&AYU_countMutex);

  uint64_t ts = AYU_rdtsc ();	// timestamp [cycles]

  std::string bt_file;

  buf[0] = htobe64 (runtimeId);
  buf[1] = htobe64 (taskId);
  buf[2] = htobe64 (event);
  buf[3] = htobe64 (0);
  buf[4] = htobe64 (0);
  buf[5] = htobe64 (0);
  buf[6] = htobe64 (0);
  buf[7] = htobe64 (ts);

  // Identification --------------
#if AYU_VERBOSE > 1
  if (event != AYU_PRESELECTTASK)
    {
      printf("%d POSIXID %x\n",ayu_event_count,pthread_self ());
    }
#endif

  // return;
  // -----------------------------

  // initialise ---------
  if (event == AYU_PREINIT)
    {
#if AYU_VERBOSE > -1
      printf("AYUDAME: initialising. version %s\n",AYU_VERSION);
      if (AYU_CONNECTTOSOCKET){
	printf("AYUDAME: will connect to socket\n");
      }
      if (AYU_WRITETOSTDOUT){
	printf("AYUDAME: will write to stdout\n");
      }
#endif
	  if ( p == NULL ){
		runtimeId = AYU_RT_SMPSS ;
	  }
	  else{
     	 runtimeId = *(ayu_runtime_t *) p;
	  }
#if AYU_CONNECTTOSOCKET

      c = AYU_initSocket ();
      if (c < 0)
	{
	  fprintf(stderr,"AYUDAME: ERROR: AYU_initSocket() failed.\n");
	  return;
	}

      // locking for future purpose of disconnect and reconnect to
      // (several) client(s)
      pthread_mutex_lock (&AYU_clientMutex);
      c = AYU_client;
      pthread_mutex_unlock (&AYU_clientMutex);

      buf[0] = htobe64 (runtimeId);

      pthread_mutex_lock (&AYU_msgMutex);
      AYU_write (c, (void *) buf, sizeof (buf));
      pthread_mutex_unlock (&AYU_msgMutex);

      AYU_initControl (c);

#endif
	  
      return;
    }
  // --------------------

#if AYU_CONNECTTOSOCKET
  // get client handle -------
  pthread_mutex_lock (&AYU_clientMutex);
  c = AYU_client;
  pthread_mutex_unlock (&AYU_clientMutex);
  // --------------------
#endif

  // wait on initialisation -------
  for (int i = 0; i < 1000; ++i)
    {
      pthread_mutex_lock (&AYU_clientMutex);
      c = AYU_client;
      pthread_mutex_unlock (&AYU_clientMutex);      

      if ( c > 0 ) {break;}
      if ( runtimeId != AYU_RT_UNKNOWN ) {break;}
      
      fprintf(stderr, "AYUDAME: waiting for init.\n");
      usleep (1.e3);
    }

#if AYU_CONNECTTOSOCKET
  if (c < 0)
    {
      fprintf(stderr, "AYUDAME: ERROR: no client handle.\n");
      return;
    }
#endif

  if (runtimeId == AYU_RT_UNKNOWN)
    {
      fprintf(stderr, "AYUDAME: ERROR: runtime unknown\n");
      return;
    }

  // set the runtime id after making sure, that PREINIT happened
  buf[0] = htobe64 (runtimeId);
  // --------------------

  // catch AYU_INIT ----
  if (event == AYU_INIT){
	if ( p == NULL ){ // for backward compatibility
		AYU_n_threads = 0;
	}
	else { 
		AYU_n_threads = *(unsigned long*)p;
	}
    if (AYU_n_threads == 0){
      char *ayu_n_threads_str;
      ayu_n_threads_str = getenv ("CSS_NUM_CPUS");
      if (ayu_n_threads_str != NULL){
	AYU_n_threads = atoi (ayu_n_threads_str);
#if AYU_VERBOSE > -1
	printf("AYUDAME: found CSS_NUM_CPUS = %ld\n",AYU_n_threads);
#endif
      }
      else {
	fprintf(stderr,"AYUDAME: number of threads could not be set\n");
      }
    }
    else {
#if AYU_VERBOSE > -1
	printf("AYUDAME: number of CPUs from runtime = %ld\n",AYU_n_threads);
#endif      
    }
    AYU_max_threads = AYU_n_threads;
  }
  // --------------------

  // catch AYU_PRESELECTTASK ----
  if (event == AYU_PRESELECTTASK)
    {
#if AYU_VERBOSE > 1
      fprintf(stderr,"AYUDAME: WARNING: Ignored event AYU_PRESELECTTASK\n");
#endif
      return;
      
    }  
  // --------------------

  // catch AYU_REGISTERFUNCTION -------

  if (event == AYU_REGISTERFUNCTION)
    {

      std::string tmp = (char const *) p;
      buf[3] = htobe64 (tmp.size ());
      buf[4] = htobe64 (regfunc_count);
      regfunc_count++;

      memset (string_buf, '\0', string_buf_size);
      strcpy (string_buf, (char const *) p);

      pthread_mutex_lock (&AYU_msgMutex);
      n = AYU_write (c, (void *) buf, sizeof (buf));
      if (n < 0){
	return;
      }
      n = AYU_write (c, (void *) string_buf, tmp.size (), true);
      if (n < 0){
	return;
      }
      pthread_mutex_unlock (&AYU_msgMutex);

      return;
    }
  // -------------------------------------------------------

  // write to client buffer and send message ---------------

  // -- AYU_INIT -- //
  if (event == AYU_INIT){
    buf[3] = htobe64 (AYU_n_threads);
  }

  // -- AYU_ADDTASK -- //
  else if (event == AYU_ADDTASK){
    int64_t *Ayu_data = (intptr_t *) p;
    buf[3] = htobe64 (Ayu_data[0]); // tasks function ID (-1 for hidden tasks)
    buf[4] = htobe64 (Ayu_data[1]); // pass whether critical

    // backtrace ------------------

#if AYU_BACKTRACE == 1
    void *bt_buffer[1000];
    char **bt_strings;
    int bt_nptrs = backtrace(bt_buffer, 1000); // number of addresses
    bt_strings = backtrace_symbols(bt_buffer, bt_nptrs);
    
    // depth 2 of calltree:
    // 0 is AYU_event, 
    // 1 is runtime call function, 
    // 2 is actual taskified function
    std::string bt_s(bt_strings[2]);
    
    std::string bt_prog = bt_s.substr(0,bt_s.find('('));
    std::string bt_addr = bt_s.substr(bt_s.find('[')+1,8);

    // get file and line with addr2line
    std::string bt_exec("addr2line -e " + bt_prog + " -f -p " + bt_addr);
    FILE *bt_out = popen(bt_exec.c_str(),"r");
    char bt_buf2[1000];
    fgets(bt_buf2, 1000, bt_out);
    pclose(bt_out);

    strtok(bt_buf2,"\n"); // strips trailing \n . is it thread safe???

    bt_file = bt_buf2;
    size_t tmp = bt_file.find(':');
    size_t tmp2 = bt_file.find_last_of(' ')+1;
    
    // This gets the calling function:
    std::string bt_func = bt_file.substr(0,bt_file.find(' '));
    
    // This gets the line number:
    std::string bt_line_no = bt_file.substr(tmp+1);
    
    // This gets the file name:
    bt_file = bt_file.substr(tmp2,tmp-tmp2);

    // To get the source code line:
    //    bt_exec = "sed '" + bt_line_no + "!d' " + bt_file; // print one line of file
    //    bt_out = popen(bt_exec.c_str(),"r");
    //    fgets(bt_buf2, 1000, bt_out);
    //    pclose(bt_out);
    //    strtok(bt_buf2,"\n"); // strips trailing \n . is it thread safe???


    //Backtrace_symbols_fd(buffer, nptrs, STDOUT_FILENO);

    //      show_backtrace();

    memset (string_buf, '\0', string_buf_size); // file name
    strcpy (string_buf, bt_file.c_str());
      
    buf[5] = htobe64 (bt_file.size()); // length of file name
    buf[6] = htobe64 (atol(bt_line_no.c_str())); // line number

#endif // AYU_BACKTRACE

  }

  // -- AYU_ADDDEPENDENCY -- //
  else if (event == AYU_ADDDEPENDENCY) {
    uintptr_t *Ayu_data = (uintptr_t *) p;
    buf[3] = htobe64 (Ayu_data[0]);	// pass task ID of last writer
    buf[4] = htobe64 (Ayu_data[1]);	// dependency address
    buf[5] = htobe64 (Ayu_data[2]);	// original dependency address
  }

  // -- AYU_PRERUNTASK -- //
  else if (event == AYU_PRERUNTASK) {
    buf[3] = htobe64 (*(int *) p);	// thread Id
  }

  // -- AYU_ADDTASKTOQUEUE -- //
  else if (event == AYU_ADDTASKTOQUEUE) {
    buf[3] = htobe64 (*(int *) p);	// thread Id
  }

  pthread_mutex_lock (&AYU_msgMutex);
  n = AYU_write (c, (void *) buf, sizeof (buf));
  if (n < 0){
    return;
  }
  
#if AYU_BACKTRACE == 1
  if (event == AYU_ADDTASK){
    n = AYU_write (c, (void *) string_buf, bt_file.size(), true );
    if (n < 0){
      return;
    }
  }
#endif // BACKTRACE

  pthread_mutex_unlock (&AYU_msgMutex);
  
  /*
#if AYU_WRITETOSTDOUT
  printf("AYUDAME_EVENT: "); 
  for (size_t i=0; i<buf_size; ++i){
    printf(" %d",be64toh(buf[i])); 
  }
  printf("\n");
#if AYU_BACKTRACE == 1
  printf("%s %d\n",string_buf,bt_file.size());;
  
#endif // BACKTRACE
#endif // AYU_WRITETOSTDOUT
  */
  // --------------------

#if AYU_CONNECTTOSOCKET

  if (event == AYU_INIT ||
      event == AYU_PRERUNTASK ||
      event == AYU_REMOVETASK || 
      event == AYU_WAITON || 
      event == AYU_BARRIER)
    {
      AYU_pause (event, taskId);
    }

#endif

  return;

}

/******/

/****f* AYUDAME/AYU_initSocket
 *  NAME
 *    AYU_initSocket
 *    - initialise server side communication for the AYUDAME tool
 *  SYNOPSIS
 *    AYU_initSocket()
 *  DESCRIPTION
 *    Initialises the server side communication for the AYUDAME tool
 *    performing the following tasks:
 *    * check for the environment variable AYU_PORT
 *    * create a socket
 *    * bind to the port AYU_PORT
 *    * listen for a client to connect
 *    * accept the client
 *    * set AYU_client to the client handle
 *  PARAMETERS
 *    void
 *  RETURN VALUE
 *    returns 0 on success and -1 on failure
 *  TODO
 *    * possibility for various clients to connect
 *  SOURCE
 */

int
AYU_initSocket (void)
{
  // declare variables -----------------------

  int server_socket;		// server socket handle
  int client_socket;		// client socket handle
  int cli_size;			// client socket size
  struct sockaddr_in srv, cli;	// address structs for srv and cli sockets
  unsigned short int ayu_socket_port;	// port to which the socket will
  // connect

  // -----------------------------------------

  // initialise variables --------------------

  server_socket = client_socket = cli_size = -1;
  memset (&srv, 0, sizeof (struct sockaddr_in));
  memset (&cli, 0, sizeof (struct sockaddr_in));
  ayu_socket_port = 0;

  // -----------------------------------------

  // read AYU_PORT from environment ---
  char *ayu_socket_port_str;
  ayu_socket_port_str = getenv ("AYU_PORT");
  if (ayu_socket_port_str != NULL)
    {
      ayu_socket_port = atoi (ayu_socket_port_str);
#if AYU_VERBOSE > -1
      printf("AYUDAME: found AYU_PORT = %d\n",ayu_socket_port);
#endif
    }
  else
    {
      ayu_socket_port = getpid () % 1000 + 5000;
#if AYU_VERBOSE > -1
      printf("AYUDAME: AYU_PORT set to %d\n",ayu_socket_port);
#endif
    }
  // -----------------------------------------

  // establish socket ------------------------
  // TODO: Do we need IPv6 support?
  server_socket = socket (AF_INET, SOCK_STREAM, 0);
  if (server_socket == -1)
    {
      perror ("Socket creation failed");
      return -1;
    }

  srv.sin_addr.s_addr = INADDR_ANY;
  srv.sin_port = htons (ayu_socket_port);
  srv.sin_family = AF_INET;

  if (bind (server_socket, (struct sockaddr *) &srv, sizeof (srv)) == -1)
    {
      char error_msg[128];
      snprintf (error_msg, 127,
		"Bind socket to ayudame port %u failed.", ayu_socket_port);
      perror ((const char *) error_msg);
      return -1;
    }
  // -----------------------------------------

  // listen and accept client -----------------------
  if (listen (server_socket, 1) == -1)
    {
      perror ("listen() failed");
      return -1;
    }

  cli_size = sizeof (cli);
  client_socket =
    accept (server_socket, (struct sockaddr *) &cli,
	    (socklen_t *) & cli_size);

  if (client_socket == -1)
    {
      perror ("accept() failed");
      return -1;
    }
#if AYU_VERBOSE > 0
  printf("AYUDAME: client accepted from %s on port %d\n",inet_ntoa (cli.sin_addr),ayu_socket_port);
#endif

  // -----------------------------------------

  pthread_mutex_lock (&AYU_clientMutex);
  AYU_client = client_socket;
  pthread_mutex_unlock (&AYU_clientMutex);

  return 0;
}

/******/

/****f* AYUDAME/AYU_write
 *  NAME
 *    AYU_write
 *    - write data to a socket connection
 *  SYNOPSIS
 *    n = AYU_write(c,buf,buf_size)
 *  DESCRIPTION
 *    writes data to a client accessed by a handle
 *  PARAMETERS
 *    int c                // client handle
 *    void *buf            // pointer to data chunk
 *    size_t buf_size      // size of data chunk in bytes
 *  RETURN VALUE
 *    integer number of sent bytes, -1 on failure
 *  SOURCE
 */
int
AYU_write (int c, void *buf, size_t buf_size, bool isString)
{
#if AYU_CONNECTTOSOCKET
  int n = -1;
  for (unsigned int i = 0; i < buf_size; i += n)
    {
      n = write (c, (uint8_t *) buf + i, buf_size - i);
      if (n < 0)
  	{
  	  perror ("ERROR writing to client socket");
  	  sleep (10);
  	  return -1;
  	}
    }
#endif

#if AYU_WRITETOSTDOUT
  char outbuf[128];
  char tmpbuf[128];
  memset (outbuf, '\0', 128);
  memset (tmpbuf, '\0', 128);
  sprintf(outbuf,"AYUDAME_SEND:");
  if (isString){
    strcat(outbuf," ");
    strcat(outbuf,(char*)buf);
  }
  else{
    for (unsigned int i = 0; i < buf_size / sizeof (uint64_t); ++i)
      {
	sprintf(tmpbuf," %ld",be64toh (((uint64_t *) buf)[i]));
	strcat(outbuf,tmpbuf);
      }
  }
  printf("%s\n",outbuf);
#endif

#if AYU_CONNECTTOSOCKET
  return n;
#else
  return 0;
#endif
}

/******/

/****f* AYUDAME/AYU_rdtsc
 *  NAME
 *    AYU_rdtsc
 *    - get cycle count
 *  SYNOPSIS
 *    t = AYU_rdtsc()
 *  DESCRIPTION
 *    return timestamp in cycle counts
 *  PARAMETERS
 *    void
 *  RETURN VALUE
 *    integer number of cycles
 *  SOURCE
 */
extern "C"
{
  __inline__ uint64_t AYU_rdtsc (void)
  {
    uint32_t lo, hi;
    __asm__ __volatile__ (	// serialize
			  "xorl %%eax,%%eax \n        cpuid":::"%rax",
			  "%rbx", "%rcx", "%rdx");
    __asm__ __volatile__ ("rdtsc":"=a" (lo), "=d" (hi));
    return (uint64_t) hi << 32 | lo;
  }
}
/******/

/****f* AYUDAME/AYU_registerTask
 *  NAME
 *    AYU_registerTask
 *    - obsolete. kept for backward compatibility
 *  SYNOPSIS
 *    
 *  DESCRIPTION
 *    
 *  PARAMETERS
 *    
 *  RETURN VALUE
 *    
 *  SOURCE
 */
extern "C"
{
  void AYU_registerTask(void*)
  {
    fprintf(stderr,"AYU_registerTask is deprecated. Please delete calls.\n");
  }

}
/******/
