/****H* AYUDAME/Ayudame_types.h
 *  NAME
 *    Ayudame_types.h - header file for the event and request types of the 
 *    AYUDAME package
 *  DESCRIPTION
 *    This header file declares the following elements:
 *    * enum ayu_event_t
 *    * enum ayu_request_t
 *    * ...TODO...
 ******/

#ifndef AYUDAME_H
#define AYUDAME_H

#include <stdint.h>
#include <unistd.h>

//#define _REENTRANT // defined by default by gcc

#ifndef AYU_VERBOSE
#  define AYU_VERBOSE 0
#endif

#ifndef AYU_CONNECTTOSOCKET
#  define AYU_CONNECTTOSOCKET 1
#endif

#ifndef AYU_WRITETOSTDOUT
#  define AYU_WRITETOSTDOUT 0
#endif

#ifndef AYU_MASTER_TASKID
#  define AYU_MASTER_TASKID 0
#endif

#define AYU_VERSION "0.9.20121030"

// includes and defines for time measurement with rdtsc
#if defined(__linux__)
#  include <endian.h>
#elif defined(__FreeBSD__) || defined(__NetBSD__)
#  include <sys/endian.h>
#elif defined(__OpenBSD__)
#  include <sys/types.h>
// # define be16toh(x) betoh16(x)
// # define be32toh(x) betoh32(x)
#  define be64toh(x) betoh64(x)
#else
#  error ERROR: System not recognised!
#endif

#ifndef htobe64
#  include <byteswap.h>
#  if __BYTE_ORDER == __LITTLE_ENDIAN
#    define htobe64(x) __bswap_64 (x)
#    define be64toh(x) __bswap_64 (x)
#  else
#    define htobe64(x) (x)
#    define be64toh(x) (x)
#  endif
#endif

/*
 * #ifndef htobe64(x) #define htobe64(x) ((((uint64_t)htonl(x)) << 32) +
 * htonl(x >> 32)) #define be64toh(x) ((((uint64_t)ntohl(x)) << 32) +
 * ntohl(x >> 32)) #endif
 */
// ---


const size_t AYU_buf_size=8;       // size of message buffer
extern unsigned long AYU_n_threads;       // current number of threads
extern unsigned long AYU_max_threads;     // maximum number of threads

/****t* Ayudame_types.h/ayu_event_t
 *  NAME
 *    ayu_event_t - enum of events
 *  DESCRIPTION
 *    ayu_event_t consists of the following events:
 *    *  AYU_EVENT_NULL -- "no event", used for initialisation etc.
 *    *  AYU_PREINIT -- raised in css_preinit()
 *    *  AYU_INIT -- raised in css_init()
 *    *  AYU_FINISH -- raised in css_finish()
 *    *  AYU_REGISTERFUNCTION -- raised in css_registerTask() with the 
 *       function name as third parameter
 *    *  AYU_ADDTASK -- raised in css_addTask() after a task has been
 *       generated with the task
 *       pointer as second parameter
 *    *  AYU_ADDHIDDENTASK -- raised in HiddenWork::addPendingHiddenTask()
 *       after a hidden task has
 *       been generated with the task pointer as second parameter
 *    *  AYU_ADDWAITONTASK -- raised in css_waitOn() to signal that a waiton
 *       directive has been encountered. Note that the taskId is allways "0".
 *       This event is followed by one or more AYU_ADDDEPENDENCY events.
 *    *  AYU_ADDDEPENDENCY -- raised in ParameterConversion::convert() with
 *       task pointer as second parameter and TrackableObject (containing
 *       information about the lastWriter and the dependency address as 
 *       third parameter
 *    *  AYU_ADDTASKTOQUEUE -- raised in TaskQueues::addGlobalTask(),
 *       TaskQueues::addRRToNode, TaskQueues::addTask,
 *       TaskQueues::addTaskWithLocality, TaskQueues::addAffineTask
 *    *  AYU_PRESELECTTASK -- raised in
 *       WorkerEngine::TaskExecutor::tryToRunTaskOrWait() before a task
 *       is assigned
 *    *  AYU_PRERUNTASK -- raised in
 *       WorkerEngine::TaskExecutor::tryToRunTaskOrWait() after a task is
 *       assigned
 *    *  AYU_RUNTASK -- raised in
 *       WorkerEngine::TaskExecutor::tryToRunTaskOrWait() just before a task 
 *       is executed
 *    *  AYU_POSTRUNTASK -- raised in
 *       WorkerEngine::TaskExecutor::tryToRunTaskOrWait() after runTask()
 *    *  AYU_RUNTASKFAILED -- raised in
 *       WorkerEngine::TaskExecutor::tryToRunTaskOrWait() if Exception
 *       is raised by runTask()
 *    *  AYU_REMOVETASK -- raised in
 *       TaskGraph::removeNodeAndEnqueueFreeSuccessors() and
 *       HiddenWork::removePendingHiddenTask()
 *    *  AYU_WAITON -- raised in css_waitOn()
 *    *  AYU_BARRIER -- raised in css_barrier
 *
 *  SOURCE
 */

enum ayu_event_t {
  AYU_EVENT_NULL = 0,
  AYU_PREINIT = 1,
  AYU_INIT = 2,
  AYU_FINISH = 3,
  AYU_REGISTERFUNCTION = 4,
  AYU_ADDTASK = 5,
  AYU_ADDHIDDENTASK = 6,
  AYU_ADDDEPENDENCY = 7,
  AYU_ADDTASKTOQUEUE = 8,
  AYU_PRESELECTTASK = 9,
  AYU_PRERUNTASK = 10,
  AYU_RUNTASK = 11,
  AYU_POSTRUNTASK = 12,
  AYU_RUNTASKFAILED = 13,
  AYU_REMOVETASK = 14,
  AYU_WAITON = 15,
  AYU_BARRIER = 16,
  AYU_ADDWAITONTASK = 17
};

/******/

/****t* Ayudame_types.h/ayu_request_t
 *  NAME
 *    ayu_request_t - enum of requests
 *  DESCRIPTION
 *    ayu_event_t consists of the following events:
 *    * AYU_REQUEST_NULL -- "no request", used for initialisation
 *    * AYU_NOREQUEST -- explicit "no request"
 *    * AYU_PAUSEONEVENT -- pause request, event upon which the program should
 *      pause is given as third parameter, fourth parameter is 0 for "off"
 *      ("un-pause") and 1 for "on"
 *    * AYU_PAUSEONTASK --  pause request, task id is given as third parameter,
 *      fourth parameter is 0 for "off" ("un-pause") and 1 for "on"
 *    * AYU_PAUSEONFUNCTION -- pause request, function id is given as third
 *      parameter, fourth parameter is 0 for "off" ("un-pause") and 1 for "on"
 *    * AYU_STEP -- run until next pause condition is reached
 *    * AYU_BREAKPOINT -- set breakpoint, i.e. don't assign new tasks, third
 *      parameter is 0 for "off" ("un-pause") and 1 for "on"
 *  SOURCE
 */

enum ayu_request_t {
  AYU_REQUEST_NULL = 0,
  AYU_NOREQUEST = 1,
  AYU_PAUSEONEVENT = 2,
  AYU_PAUSEONTASK = 3,
  AYU_PAUSEONFUNCTION = 4,
  AYU_STEP = 5,
  AYU_BREAKPOINT = 6,
  AYU_BLOCKTASK = 7,
  AYU_PRIORITISETASK = 8,
  AYU_SETNUMTHREADS = 9
};
/******/

/****t* Ayudame_types.h/ayu_runtime_t
 *  NAME
 *    ayu_runtime_t - enum of runtime ids
 *  DESCRIPTION
 *    ayu_event_t consists of the following ids:
 *    * AYU_RT_UNKNOWN -- no or unknown runtime, used for initialisation
 *    * AYU_RT_CPPSS -- CPPSS runtime
 *    * AYU_RT_SMPSS -- SmpSs runtime
 *    * AYU_RT_OMPSS -- OmpSs runtime
 *    * AYU_RT_GOMP -- GNU OpenMP runtime
 *    * AYU_RT_CILK -- CILK runtime
 *  SOURCE
 */
enum ayu_runtime_t {
  AYU_RT_UNKNOWN = 0,
  AYU_RT_CPPSS = 1,
  AYU_RT_SMPSS = 2,
  AYU_RT_OMPSS = 3,
  AYU_RT_GOMP = 4,
  AYU_RT_CILK = 5
};
/******/

extern "C" {

  /****f* AYUDAME/AYU_event
   *  NAME
   *    AYU_event
   *    - callback for the *Ss runtime
   *  SYNOPSIS
   *    AYU_event(event, taskId, pointer)
   *  DESCRIPTION
   *    this function is called from the *Ss runtime to communicate
   *    with the AYUDAME package. Events of type ayu_event_t, the task ID
   *    of the calling task and a pointer to arbitrary data can be sent.
   *    AYU_event will connect to a client via tcp sockets and pass on
   *    the information.
   *  PARAMETERS
   *    ayu_event_t event      // type of event
   *    const int64_t taskId   // taskId from which the event is sent
   *    void *p                // pointer to further data
   *  RETURN VALUE
   *    void
   *  SOURCE
   */
  void AYU_event(ayu_event_t event, const int64_t taskId, void *p)
    __attribute__ ((weak));
  /******/

  /****f* AYUDAME/AYU_getBreakpoint
   *  NAME
   *    AYU_getBreakpoint
   *    - return AYU_breakpoint
   *  SYNOPSIS
   *    breakpoint = AYU_getBreakpoint()
   *  DESCRIPTION
   *    Locks the global variable AYU_breakpoint, returns its value
   *    and unlocks it
   *  PARAMETERS
   *    void
   *  RETURN VALUE
   *    1 if AYU_breakpoint is set
   *    0 if AYU_breakpoint is not set
   *  SOURCE
   */
  int AYU_getBreakpoint(void) __attribute__ ((weak));
  /******/

  /****f* AYUDAME/AYU_isBlocked
   *  NAME
   *    AYU_isBlocked
   *    - return true if task is blocked
   *  SYNOPSIS
   *    if(AYU_isBlocked(taskId)) { do_something(); }
   *  DESCRIPTION
   *    Locks the global variable AYU_blockedTask using the
   *    breakpoint mutex lock, compares to it
   *    taskId given as parameter and returns result
   *  PARAMETERS
   *    * const int taskId
   *  RETURN VALUE
   *    true if task is blocked
   *    false if task is not blocked
   *  SOURCE
   */
  bool AYU_isBlocked(const int taskId) __attribute__ ((weak));
  /******/

  /****f* AYUDAME/AYU_getPriorityLevel
   *  NAME
   *    AYU_getPriorityLevel
   *    - returns priority of task
   *  SYNOPSIS
   *    AYU_getPriorityLevel(taskId)
   *  DESCRIPTION
   *    returns the priority of a task identified by its id.
   *    If the priority has not been changed in Ayudame it
   *    returns -1.
   *  PARAMETERS
   *    int taskId
   *  RETURN VALUE
   *    -1: if priority has not changed
   *    0:  low (no) priority
   *    positive integer: priority level
   *  SOURCE
   */
  int AYU_getPriorityLevel(int taskId) __attribute__ ((weak));
  /******/

  /****f* AYUDAME/AYU_getNumThreads
   *  NAME
   *    AYU_getNumThreads
   *    - returns number of threads set by AYUDAME
   *  SYNOPSIS
   *    runtime_setNumThreads(AYU_getNumThreads());
   *  DESCRIPTION
   *    returns the number of threads as set by ayudame
   *  PARAMETERS
   *    void
   *  RETURN VALUE
   *    unsigned int: number of threads
   *  SOURCE
   */
  unsigned int AYU_getNumThreads() __attribute__ ((weak));
  /******/
  
  /****f* AYUDAME/AYU_setNumThreads
   *  NAME
   *    AYU_setNumThreads
   *    - sets number of threads in the runtime
   *  SYNOPSIS
   *    AYU_setNumThreads(n_threads);
   *  DESCRIPTION
   *    This callback function can be implemented in the StarSs
   *    runtime. If it is not implemented, the client of the
   *    AYUDAME library will not be able to change the number of 
   *    threads.
   *  PARAMETERS
   *    unsigned int n_threads
   *  RETURN VALUE
   *    void
   */
  void AYU_setNumThreads(unsigned int n_threads) __attribute__ ((weak));
  /******/
  

  /****f* AYUDAME/AYU_registerTask
   *  NAME
   *    AYU_registerTask
   *    - obsolete. kept for backward compatibility
   *  SYNOPSIS
   *    
   *  DESCRIPTION
   *    
   *  PARAMETERS
   *    
   *  RETURN VALUE
   *    
   *  SOURCE
   */
  void AYU_registerTask(void*) __attribute__ ((weak));
  /******/

}

#ifndef AYU_MASTER_TASKID
#define AYU_MASTER_TASKID 0
#endif

#endif //AYUDAME_H
