#!/usr/bin/env python

#****h* *Ss-Tools/TEMANEJO
# NAME
#   TEMANEJO
#   - display and interaction with *Ss' task graph
# DESCRIPTION
#   TEMANEJO connects as a client to a socket provided by AYUDAME.
#   It performs the following tasks:
#   * display the task graph
#   * pause the execution on user request
# USAGE
#   TEMANEJO can either connect to a running SMPSs application ("remote"),
#   which is waiting for a client or start the application itself and make all
#   the necessary adjustments ("local").
#
#   Find help on the command line option by typing
#     temanejo.py -h
#
#   -- Local connection --
#
#   To let TEMANEJO start the application run
#     temanejo.py [-l <layout>] [-d <layout_direction>]\
#                 [-v[v[v]]|-q] [-z|-Z] [smpss_app [smpss_app_args]]
#                 [-t <title>] [-n NODE_COL] [-m MARGIN_COL]
#                 [-e EDGE_COL] [-s SHAPES] [-L LIB_FILE]
#
#   <layout> is one of dot, twopi. This option
#   determines the graphical layout of the graph. We recommend to go with the
#   default 'dot'.
#
#   <layout_direction> is one of t2b, l2r, b2t, r2l and determines whether to
#   draw the graph from top to bottom, left to right and so forth.
#
#   The option -z (-Z) switches on autozoom off (on).
#
#   The options -q, -v, -vv, -vvv set the verbosity level from quiet to
#   very, very verbose.
#
#   With -t you can set a name for the window (usually displayed in the
#   title bar of the window decoration).
#
#   -n, -m, -e and -s control which information to display as node colour,
#   margin colour, edge colour and shape. Please see temanejo.py --help
#   for details.
#
#   -L will load a specific (AYUDAME) library for communication with the
#   application.
#
#   The main window and the connect dialog will open where you can edit the
#   adjustments for the application (resp. choose it if it was not passed as
#   command line argument), the ayudame library (defaults to libayudame.so in
#   the working directory), the configure file for SMPSs, the socket port
#   and the number of cores to be used. Connect by clicking "Connect".
#
#   -- Remote connection --
#
#   To let TEMANEJO connect to a running application (either remote or at
#   localhost) type:
#     temanejo.py [-l <layout>] [-d <layout_direction>] [-z]
#   See above for a description of the options
#
#   The main window and the connect dialog will open where you can edit the
#   adjustments for the host and the socket port. make sure, the application is
#   running, LD_PRELOAD was set to libayudame.so and it is waiting for a client.
#   Connect by clicking "Connect".
#
#   -- Running the application --
#
#   The execution will initially stop whan css_init() is reached, i.e. when the
#   pragma
#     #pragma css start
#   is reached. Choose one or more event on which you want to pause on using the
#   checkbuttons at the lower left of the window. Then you run until a pause
#   condition is reached by clicking on the "play" button in the toolbar.
#
# PORTABILITY
#   The following modules are imported:
#   * optparse
#   * os
#   * sys
#   * subprocess
#   * struct
#   * types
#   * gtk
#   * gtk.gdk
#   * gtk.keysyms
#   * socket
#   * threading
#   * temanejo_graph
#   * temanejo_graphWidget
# TODO
#    * connect and disconnect to AYUDAME
#    * connect to several hosts (nodes)
#    * set next task to run
# AUTHOR
#    Steffen Brinkmann, HLRS <brinkmann@hlrs.de>
# COPYRIGHT
#    (C) 2010-2012, HLRS, University of Stuttgart
#    temanejo.py is published under the terms of the BSD license.
#    See the LICENSE file for details.
#    temanejo.py uses code of xdot.py, ver. 0.4 (C) 2008 Jose Fonseca
#    which is published under the terms of the GNU Lesser General Public
#    License
#    temanejo.py uses code of NetworkX, (C) 2004-2009 by Aric Hagberg,
#    Dan Schult and Pieter Swart. Published under the BSD license.

#******

'''Visualise and interact with task dependency graphs from StarSs.'''

__progname__ = 'TEMANEJO'

__author__ = 'Steffen Brinkmann'

__version__ = '0.9.20121030'

#import cProfile

import optparse
import os
import string
import shlex
import signal
import subprocess
import struct
import sys
import tempfile
import types

try:
    import pygtk
    pygtk.require('2.0')
except:
    pass
import gtk
import gtk.gdk
import gtk.keysyms

import socket
from threading import Thread, Event #, Lock

from temanejo_graph import *
from temanejo_graphWidget import *

# how many threads to display when application is run on remote computer
THREADS_DISPLAY = 16

# base path of temanejo.py
for path in sys.path :
	if os.path.exists(os.path.join(path, "temanejo.py")):
		temanejo_base_dir = path
		break
if temanejo_base_dir == None :
	print __progname__, ': Error finding base path for this executable'


# equivalent to enum of events and requests
event_t=['AYU_EVENT_NULL', \
             'AYU_PREINIT', \
             'AYU_INIT', \
             'AYU_FINISH', \
             'AYU_REGISTERTASK',\
             'AYU_ADDTASK', \
             'AYU_ADDHIDDENTASK', \
             'AYU_ADDDEPENDENCY', \
             'AYU_ADDTASKTOQUEUE', \
             'AYU_PRESELECTTASK', \
             'AYU_PRERUNTASK', \
             'AYU_RUNTASK', \
             'AYU_POSTRUNTASK', \
             'AYU_RUNTASKFAILED', \
             'AYU_REMOVETASK', \
             'AYU_WAITON', \
             'AYU_BARRIER', \
             'AYU_ADDWAITONTASK']

request_t=['AYU_REQUEST_NULL', \
               'AYU_NOREQUEST', \
               'AYU_PAUSEONEVENT', \
               'AYU_PAUSEONTASK', \
               'AYU_PAUSEONFUNCTION', \
               'AYU_STEP', \
               'AYU_BREAKPOINT', \
               'AYU_BLOCKTASK', \
               'AYU_PRIORITISETASK',\
               'AYU_SETNUMTHREADS']

#****c* TEMANEJO/TemanejoWindow
# NAME
#   TemanejoWindow
#   - python class for the window of TEMANEJO
# DESCRIPTION
#   This represents the main window and user interface of TEMANEJO.
# DERIVED FROM
#   gtk.Window
# METHODS
#   __init__
#   cleanup
#   get_active_text
#   highlight_checkbox
#   on_about
#   on_clearmarked
#   on_connect
#   on_controlcheckbutton
#   on_debug
#   on_debugfunction
#   on_export
#   on_gototask
#   on_margcollist
#   on_nodecollist
#   on_quit
#   on_refresh
#   on_run
#   on_run_nsteps
#   on_settings
#   on_shapelist
#   push_statusbar
#   register_function
#   start_gdb
#   update_ui
# ATTRIBUTES
#   ui
# SOURCE

class TemanejoWindow(gtk.Window):

    ui = '''
    <ui>
        <toolbar name='ToolBar'>
            <toolitem action='Connect'/>
            <toolitem action='Settings'/>
            <separator/>
            <toolitem action='Refresh'/>
            <toolitem action='ZoomIn'/>
            <toolitem action='ZoomOut'/>
            <toolitem action='ZoomFit'/>
            <toolitem action='Zoom100'/>
            <separator/>
            <toolitem action='Run'/>
            <toolitem action='Run_fast' name='Run_fast'/>
            <toolitem action='Debug'/>
            <separator />
            <toolitem action='Export'/>
            <separator name='Sep'/>
            <toolitem action='About'/>
            <toolitem action='Quit'/>
        </toolbar>
    </ui>
    '''
#            <toolitem action='Help'/>
#            <toolitem action='Breakpoint'/>
#            <toolitem action='Step'/>

#<toolitem action='Open'/>

    def __init__(self):
        # ---
        gtk.Window.__init__(self)

        window = self
        window.set_default_size(1280, 768)
        window.set_icon_from_file(os.path.join(temanejo_base_dir,'resources/temanejo_icon.png'))

        self.is_fullscreen = False

        # use this line to change the application wide font: 
        #self.get_settings().set_string_property('gtk-font-name', 'sans normal 10','');

        # Graph and Graph widget
        self.graph = Graph()
        self.widget = GraphWidget(self.graph,self)
        #self.graph.set_widget(self.widget)

        # string holding executable and its arguments
        self.executable = None
        self.executable_args = ''
        self.exec_cmd = ''

        # string holding ayudame library
        self.library=os.path.join(temanejo_base_dir,'./lib/libayudame.so')
        if not os.path.isfile(self.library):
            self.library=None

        # last event and taskId for status bar; blocked task Id
        self.last_event='AYU_EVENT_NULL'
        self.last_check_event='AYU_EVENT_NULL'
        self.last_taskId=-1
#        self.blockedTask=0

        # how many threads to display in list
        self.threads_dis = THREADS_DISPLAY

        # current number of threads
        #self.numthreads = 2

        # how many steps to run when running several steps
        self.n_steps=10

        # communticator thread and socket
        self.comm=None
        self.sock=None

        # thread for execution of the application
        self.execut=None

        # barrier counter
        self.barrier_count = 0

        # number of registered functions
        self.func_count=0

        # counters
        self.func_counter    ={} # total
        self.func_counter_NQ ={} # not queued
        self.func_counter_Q  ={} # queued
        self.func_counter_R  ={} # running
        self.func_counter_F  ={} # finished
#        self.func_counter_X  ={} # failed

        self.thread_counter    ={} # total
#        self.thread_counter_NQ =0 # not queued
        self.thread_counter_Q  ={} # queued
        self.thread_counter_R  ={} # running
        self.thread_counter_F  ={} # finished
#        self.thread_counter_X  ={} # failed

        self.status_counter_NQ =0 # not queued
        self.status_counter_Q  =0 # queued
        self.status_counter_R  =0 # running
        self.status_counter_F  =0 # finished
#        self.status_counter_X  =0 # failed

        # Create a UIManager instance
        uimanager = self.uimanager = gtk.UIManager()

        # Add the accelerator group to the toplevel window
        accelgroup = uimanager.get_accel_group()
        window.add_accel_group(accelgroup)

        # Create an ActionGroup
        self.actiongroup = gtk.ActionGroup('Actions')

        # Create actions
        self.actiongroup.add_actions([
                #('Open', gtk.STOCK_OPEN, None, None, None, self.on_open),
                ('Connect', gtk.STOCK_CONNECT, None, None, 'Connect application (c)', self.on_connect),
                ('Settings', gtk.STOCK_PREFERENCES, None, None, 'Preferences (p)', self.on_settings),
                ('Refresh', gtk.STOCK_REFRESH, None, None, 'Refresh the view (r)', self.on_refresh),
                ('ZoomIn', gtk.STOCK_ZOOM_IN, None, None, 'Zoom in (+,PageUp)', self.widget.on_zoom_in),
                ('ZoomOut', gtk.STOCK_ZOOM_OUT, None, None, 'Zoom out (-,PageDown)', self.widget.on_zoom_out),
                ('ZoomFit', gtk.STOCK_ZOOM_FIT, None, None, 'Zoom to fit (Escape)', self.widget.on_zoom_fit),
                ('Zoom100', gtk.STOCK_ZOOM_100, None, None, 'Default zoom level (z)', self.widget.on_zoom_100),
                ('About', gtk.STOCK_DIALOG_INFO, 'About', None, 'About TEMANEJO (i)', self.on_about),
                ('Export', gtk.STOCK_CONVERT, 'Export', None, 'Export Graph in different file formats (e)', self.on_export),
#                ('Help', gtk.STOCK_DIALOG_QUESTION, 'Help', 'h', 'Get help', self.on_help),
                ('Quit', gtk.STOCK_QUIT, None, None, 'Quit TEMANEJO (Ctrl+q)', self.on_quit),
                ('Run', gtk.STOCK_MEDIA_PLAY, 'Run', None, 'Run program to next pause event (space)', self.on_run),
                ('Run_fast', gtk.STOCK_MEDIA_FORWARD, 'Run '+str(self.n_steps)+' steps', None,
                 'Run program and ignore several pause events. Change the number of events in the preferences. (Ctrl+space)', self.on_run_nsteps),
                ('Debug', gtk.STOCK_GO_FORWARD, 'Debug', None, 'Debug with gdb (d)', self.on_debug)
                ])
# interesting stock button icons: GOTO_LAST,EXECUTE, GO_FORWARD, GOTO_LAST, STOP, MEDIA_PLAY, MEDIA_STOP, MEDIA_PAUSE

        self.actiongroup.get_action('Run_fast').set_sensitive(False)

        # Add the actiongroup to the uimanager
        uimanager.insert_action_group(self.actiongroup, 0)

        # Add a UI descrption
        uimanager.add_ui_from_string(self.ui)

        # Create a toolbar
        self.toolbar = uimanager.get_widget('/ToolBar')

        # Adjust the separator
        sep = uimanager.get_widget('/ToolBar/Sep')
        sep.set_expand(True)
        sep.set_draw(False)

        # Create status bars
        self.statusbar = gtk.Statusbar()
        self.statusbar.set_has_resize_grip(False)
        self.statusbar_lastmsg_cid = self.statusbar.get_context_id('last_msg')
        self.statusbar_lastmsg_id=self.statusbar.push(self.statusbar_lastmsg_cid,'No last message')

        self.statusbar2 = gtk.Statusbar()
        self.statusbar2.set_has_resize_grip(True)
        self.statusbar2_lastmsg_cid = self.statusbar2.get_context_id('last_msg')
        self.statusbar2_lastmsg_id = self.statusbar2.push(self.statusbar2_lastmsg_cid,'0 nodes, 0 edges')

        hbox_statusbars = gtk.HBox()
        hbox_statusbars.pack_start(self.statusbar)
        hbox_statusbars.pack_end(self.statusbar2)

        # Create node colour lists (treeview)

        self.label_nodecollist = gtk.Label('Node colour means')
        self.combo_nodecollist = gtk.combo_box_new_text()
        self.combo_nodecollist.append_text('function')
        self.combo_nodecollist.append_text('thread')
        self.combo_nodecollist.append_text('task status')
        self.combo_nodecollist.append_text('task duration')
        self.combo_nodecollist.append_text('distance to marked')
        self.combo_nodecollist.append_text('nothing')
        self.combo_nodecollist.set_active(0)

        hbox_nodecollist = gtk.HBox()
        hbox_nodecollist.pack_start(self.label_nodecollist)
        hbox_nodecollist.pack_start(self.combo_nodecollist)

        self.liststore_nodecol = gtk.ListStore(str, str, str, str, str, str, str, str)

        self.treeview_nodecol = gtk.TreeView(self.liststore_nodecol)
        self.tvnc_col0  = gtk.TreeViewColumn('function')
        self.tvnc_col1  = gtk.TreeViewColumn('NQ')
        self.tvnc_col2  = gtk.TreeViewColumn('Q ')
        self.tvnc_col3  = gtk.TreeViewColumn('R ')
        self.tvnc_col4  = gtk.TreeViewColumn('F ')
#        self.tvnc_col5  = gtk.TreeViewColumn('X ')
        self.tvnc_col6  = gtk.TreeViewColumn('total')

        self.treeview_nodecol.append_column(self.tvnc_col0)
        self.treeview_nodecol.append_column(self.tvnc_col1)
        self.treeview_nodecol.append_column(self.tvnc_col2)
        self.treeview_nodecol.append_column(self.tvnc_col3)
        self.treeview_nodecol.append_column(self.tvnc_col4)
#        self.treeview_nodecol.append_column(self.tvnc_col5)
        self.treeview_nodecol.append_column(self.tvnc_col6)

        self.cell = gtk.CellRendererText()

#        self.cell.set_property('cell-background-gdk', gtk.gdk.Color(.5,.8,.7))
        self.tvnc_col0.pack_start(self.cell, True)
        self.tvnc_col0.set_attributes(self.cell, text=0, cell_background=1)
        self.cell = gtk.CellRendererText()
#        self.cell.set_property('cell-background', 'yellow')
        self.tvnc_col1.pack_start(self.cell, True)
        self.tvnc_col1.set_attributes(self.cell, text=2)
        self.tvnc_col2.pack_start(self.cell, True)
        self.tvnc_col2.set_attributes(self.cell, text=3)
        self.tvnc_col3.pack_start(self.cell, True)
        self.tvnc_col3.set_attributes(self.cell, text=4)
        self.tvnc_col4.pack_start(self.cell, True)
        self.tvnc_col4.set_attributes(self.cell, text=5)
#        self.tvnc_col5.pack_start(self.cell, True)
#        self.tvnc_col5.set_attributes(self.cell, text=6)
        self.tvnc_col6.pack_start(self.cell, True)
        self.tvnc_col6.set_attributes(self.cell, text=7)

#        self.liststore_nodecol.append(['total', gtk.gdk.Color(255,255,255), '0', '0', '0', '0', '0'])
        self.liststore_nodecol.append(['total', 'white', '0', '0', '0', '0', '0', '0'])

        scroll_nodecollist = gtk.ScrolledWindow()
        scroll_nodecollist.add(self.treeview_nodecol)
        scroll_nodecollist.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)

        # Create margin colour lists (treeview)

        self.label_margcollist = gtk.Label('Margin colour means')
        self.combo_margcollist = gtk.combo_box_new_text()
        self.combo_margcollist.append_text('task status')
        self.combo_margcollist.append_text('function')
        self.combo_margcollist.append_text('thread')
        self.combo_margcollist.append_text('task duration')
        self.combo_margcollist.append_text('nothing')
        self.combo_margcollist.set_active(0)

        hbox_margcollist = gtk.HBox()
        hbox_margcollist.pack_start(self.label_margcollist)
        hbox_margcollist.pack_start(self.combo_margcollist)

        self.liststore_margcol = gtk.ListStore(str, str, str, str, str, str, str, str)

        self.treeview_margcol = gtk.TreeView(self.liststore_margcol)
        self.tvmc_col0  = gtk.TreeViewColumn('task status')
        self.tvmc_col1  = gtk.TreeViewColumn('NQ')
        self.tvmc_col2  = gtk.TreeViewColumn('Q ')
        self.tvmc_col3  = gtk.TreeViewColumn('R ')
        self.tvmc_col4  = gtk.TreeViewColumn('F ')
#        self.tvmc_col5  = gtk.TreeViewColumn('X ')
        self.tvmc_col6  = gtk.TreeViewColumn('#')

        self.treeview_margcol.append_column(self.tvmc_col0)
        self.treeview_margcol.append_column(self.tvmc_col6)

        self.cell = gtk.CellRendererText()

#        self.cell.set_property('cell-background-gdk', gtk.gdk.Color(.5,.8,.7))
        self.tvmc_col0.pack_start(self.cell, True)
        self.tvmc_col0.set_attributes(self.cell, text=0, cell_background=1)
        self.cell = gtk.CellRendererText()
#        self.cell.set_property('cell-background', 'yellow')
        self.tvmc_col1.pack_start(self.cell, True)
        self.tvmc_col1.set_attributes(self.cell, text=2)
        self.tvmc_col2.pack_start(self.cell, True)
        self.tvmc_col2.set_attributes(self.cell, text=3)
        self.tvmc_col3.pack_start(self.cell, True)
        self.tvmc_col3.set_attributes(self.cell, text=4)
        self.tvmc_col4.pack_start(self.cell, True)
        self.tvmc_col4.set_attributes(self.cell, text=5)
#        self.tvmc_col5.pack_start(self.cell, True)
#        self.tvmc_col5.set_attributes(self.cell, text=6)
        self.tvmc_col6.pack_start(self.cell, True)
        self.tvmc_col6.set_attributes(self.cell, text=7)

#        self.liststore_margcol.append(['total', gtk.gdk.Color(255,255,255), '0', '0', '0', '0', '0'])
        self.liststore_margcol.append(['not queued', 'red', '0', '0', '0', '0', '0', '0'])
        self.liststore_margcol.append(['queued', 'yellow', '0', '0', '0', '0', '0', '0'])
        self.liststore_margcol.append(['running', 'green', '0', '0', '0', '0', '0', '0'])
        self.liststore_margcol.append(['finished', 'white', '0', '0', '0', '0', '0', '0'])
        self.liststore_margcol.append(['failed', 'gray', '0', '0', '0', '0', '0', '0'])
        self.liststore_margcol.append(['total', 'white', '0', '0', '0', '0', '0', '0'])

        scroll_margcollist = gtk.ScrolledWindow()
        scroll_margcollist.add(self.treeview_margcol)
        scroll_margcollist.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)

        # Create node shape lists (treeview)

        # to be synchronised with self.graph.shape_list!!!
        self.shape_list='square', 'triangle', 'diamond',\
            'house', 'trapezium', 'pentagram', 'pentagon', 'hexagon',\
            'invtriangle', 'invhouse', 'invtrapezium', 'invpentagon',\
            'circle', 'circle', 'circle', 'circle'

        self.pb_list=[]
        for i in range(len(self.shape_list)):
            self.pb_list.append(gtk.gdk.pixbuf_new_from_file_at_size(
                    os.path.join(temanejo_base_dir,'resources/%s_100.png'%self.shape_list[i]),
                                25,25))
        self.pb_list.append(gtk.gdk.pixbuf_new_from_file_at_size(
                os.path.join(temanejo_base_dir,'resources/arrow_100.png'),
                            25,25))

        self.label_shapelist = gtk.Label('Node shape means')
        self.combo_shapelist = gtk.combo_box_new_text()
        self.combo_shapelist.append_text('thread')
        self.combo_shapelist.append_text('function')
        self.combo_shapelist.append_text('nothing')
        self.combo_shapelist.set_active(0)

        hbox_shapelist = gtk.HBox()
        hbox_shapelist.pack_start(self.label_shapelist)
        hbox_shapelist.pack_start(self.combo_shapelist)

        self.liststore_shape = gtk.ListStore(gtk.gdk.Pixbuf, str, str, str, str, str, str, str)

        self.treeview_shape = gtk.TreeView(self.liststore_shape)
        self.tvns_col0  = gtk.TreeViewColumn('shape')
        self.tvns_col7  = gtk.TreeViewColumn('thread')
        self.tvns_col1  = gtk.TreeViewColumn('NQ')
        self.tvns_col2  = gtk.TreeViewColumn('Q ')
        self.tvns_col3  = gtk.TreeViewColumn('R ')
        self.tvns_col4  = gtk.TreeViewColumn('F ')
#        self.tvns_col5  = gtk.TreeViewColumn('X ')
        self.tvns_col6  = gtk.TreeViewColumn('total')

        self.treeview_shape.append_column(self.tvns_col0)
        self.treeview_shape.append_column(self.tvns_col7)
        self.treeview_shape.append_column(self.tvns_col1)
        self.treeview_shape.append_column(self.tvns_col2)
        self.treeview_shape.append_column(self.tvns_col3)
        self.treeview_shape.append_column(self.tvns_col4)
#        self.treeview_shape.append_column(self.tvns_col5)
        self.treeview_shape.append_column(self.tvns_col6)

        self.cell = gtk.CellRendererPixbuf()

        self.tvns_col0.pack_start(self.cell, True)
        self.tvns_col0.set_attributes(self.cell, pixbuf=0)
        self.cell = gtk.CellRendererText()
        self.tvns_col7.pack_start(self.cell, True)
        self.tvns_col7.set_attributes(self.cell, text=1)
        self.tvns_col1.pack_start(self.cell, True)
        self.tvns_col1.set_attributes(self.cell, text=2)
        self.tvns_col2.pack_start(self.cell, True)
        self.tvns_col2.set_attributes(self.cell, text=3)
        self.tvns_col3.pack_start(self.cell, True)
        self.tvns_col3.set_attributes(self.cell, text=4)
        self.tvns_col4.pack_start(self.cell, True)
        self.tvns_col4.set_attributes(self.cell, text=5)
#        self.tvns_col5.pack_start(self.cell, True)
#        self.tvns_col5.set_attributes(self.cell, text=6)
        self.tvns_col6.pack_start(self.cell, True)
        self.tvns_col6.set_attributes(self.cell, text=7)

        self.liststore_shape.append([None, 'total', '0', '0', '0', '0', '0', '0'])

        scroll_shapelist = gtk.ScrolledWindow()
        scroll_shapelist.add(self.treeview_shape)
        scroll_shapelist.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)

        #### Create Control Panel

        # create checkboxes

        self.check_stopontask     = gtk.CheckButton('Stop before running task')
#        self.check_debugfunction = gtk.CheckButton('Debug function')
        self.label_debugfunction = gtk.Label('Debug function')
        self.check_anyEvent       = gtk.CheckButton('Stop on any event')
        self.check_preselecttask  = gtk.ToggleButton('Stop scheduling tasks')
        self.check_preruntask     = gtk.CheckButton('Stop before running any task')
        self.check_removetask     = gtk.CheckButton('Stop after running any task')
        self.check_waiton         = gtk.CheckButton('Stop on "wait on" directive')
        self.check_barrier        = gtk.CheckButton('Stop on barrier')
        self.eventbox_anyEvent      = gtk.EventBox()
        self.eventbox_preruntask    = gtk.EventBox()
        self.eventbox_removetask    = gtk.EventBox()
        self.eventbox_waiton        = gtk.EventBox()
        self.eventbox_barrier       = gtk.EventBox()
        self.eventbox_anyEvent.add(self.check_anyEvent)
        self.eventbox_preruntask.add(self.check_preruntask)
        self.eventbox_removetask.add(self.check_removetask)
        self.eventbox_waiton.add(self.check_waiton)
        self.eventbox_barrier.add(self.check_barrier)

        # create frame for 'any events'

        frame_breakpoints=gtk.Frame()
        frame_breakpoints.add(self.eventbox_anyEvent)

        # for convenience
        self.check_preruntask.set_active(True)

        # get default style
        self.check_default_style=self.check_anyEvent.get_modifier_style()

        # create label and combo (dropdown) box with button

        self.label_stopontask = gtk.Label('-none-')

        self.combo_debugfunction = gtk.combo_box_new_text()
        self.combo_debugfunction.append_text('-none-')
        self.combo_debugfunction.set_active(0)

        self.button_debugfunction = gtk.Button('>')

        # hboxes for stopontask and debugfunction

        hbox_stopontask = gtk.HBox()
        hbox_stopontask.pack_start(self.check_stopontask)
        hbox_stopontask.pack_start(self.label_stopontask)
        hbox_debugfunction = gtk.HBox()
        hbox_debugfunction.pack_start(self.button_debugfunction)
        hbox_debugfunction.pack_start(self.label_debugfunction)
        hbox_debugfunction.pack_start(self.combo_debugfunction)

        # create control panel

        self.control_panel  = gtk.Table(8)
        self.control_panel.attach(self.check_preselecttask,0,1,0,1)
#        self.control_panel.attach(frame_breakpoints,0,1,1,2)
        self.control_panel.attach(self.eventbox_preruntask,0,1,2,3)
#        self.control_panel.attach(self.eventbox_removetask,0,1,3,4)
        self.control_panel.attach(self.eventbox_waiton,0,1,4,5)
#        self.control_panel.attach(self.eventbox_barrier,0,1,5,6)
        self.control_panel.attach(hbox_stopontask,0,1,6,7)
        self.control_panel.attach(hbox_debugfunction,0,1,7,8)
        self.control_panel.set_border_width(10)

        # create frame for breakpoints control panel

        frame_breakpoints=gtk.Frame('Breakpoints')
        frame_breakpoints.add(self.control_panel)

        # create navigation buttons etc.

        self.button_goto_task = gtk.Button('Goto task')
        self.entry_goto_task = gtk.Entry()
        self.button_clear_marked = gtk.Button('Clear marked nodes and edges')

        # create navigation panel

        self.navigation_panel  = gtk.Table(1,2)
        self.navigation_panel.attach(self.button_goto_task,0,1,0,1)
        self.navigation_panel.attach(self.entry_goto_task,1,2,0,1)
        self.navigation_panel.attach(self.button_clear_marked,0,2,1,2)
        self.navigation_panel.set_border_width(10)
        self.navigation_panel.set_row_spacings(5)
        self.navigation_panel.set_col_spacings(5)

        # create frame for navigation control panel

        frame_navigation=gtk.Frame('Navigation')
        frame_navigation.add(self.navigation_panel)

        # create control frame

        self.button_setnumthreads = gtk.Button('Set number of threads')
        self.button_setnumthreads.set_size_request(220,-1)

        adjust = gtk.Adjustment(self.threads_dis,1,THREADS_DISPLAY,1)
        self.spin_numthreads=gtk.SpinButton(adjust,1,0)

        hbox1_control = gtk.HBox()
        hbox1_control.pack_start(self.button_setnumthreads,True)
        hbox1_control.pack_start(self.spin_numthreads)

        frame_control=gtk.Frame('Control')
        frame_control.add(hbox1_control)

        # connect widgets

        self.button_goto_task.connect('clicked',self.on_gototask, 'gototask')
        self.entry_goto_task.connect('key-release-event',self.on_gototask, 'gototask')
        self.button_clear_marked.connect('clicked',self.on_clearmarked, 'clearmarked')

        self.check_anyEvent.connect('clicked',self.on_controlcheckbutton, 'anyEvent')
        self.check_preselecttask.connect('clicked',self.on_controlcheckbutton, 'preselecttask')
        self.check_preruntask.connect('clicked',self.on_controlcheckbutton, 'preruntask')
        self.check_removetask.connect('clicked',self.on_controlcheckbutton, 'removetask')
        self.check_waiton.connect('clicked',self.on_controlcheckbutton, 'waiton')
        self.check_barrier.connect('clicked',self.on_controlcheckbutton, 'barrier')
        self.check_stopontask.connect('clicked',self.on_controlcheckbutton, 'stopontask')
        self.combo_nodecollist.connect('changed',self.on_nodecollist, 'combo_nodecollist')
        self.combo_margcollist.connect('changed',self.on_margcollist, 'combo_margcollist')
        self.combo_shapelist.connect('changed',self.on_shapelist, 'combo_shapelist')
        self.button_debugfunction.connect('clicked',self.on_debugfunction, 'debugfunction')

        self.button_setnumthreads.connect('clicked',self.on_setnumthreads, 'setnumthreads')

        # Create log window

        self.prelog = ''

        self.logbuffer = gtk.TextBuffer()
        self.logbuffer.set_text('Start session, ' + time.ctime())

        self.logview = gtk.TextView(self.logbuffer)
        self.logview.set_editable(False)
        logscroll = gtk.ScrolledWindow()
        logscroll.add(self.logview)
        #self.logview.set_size_request(-1,100)

        # Build global layout

        vbox1 = gtk.VBox()
        #vbox1.set_size_request(100,-1)
        vbox1.pack_start(hbox_nodecollist,False)
        vbox1.pack_start(scroll_nodecollist,True)
        vbox1.pack_start(hbox_margcollist,False)
        vbox1.pack_start(scroll_margcollist,True)
        vbox1.pack_start(hbox_shapelist,False)
        vbox1.pack_start(scroll_shapelist,True)

        vbox2 = gtk.VBox()
        vbox2.pack_start(frame_navigation,False,padding=5)
        vbox2.pack_start(frame_breakpoints,False,padding=5)
        vbox2.pack_start(frame_control,False,padding=5)

        #vbox3 = gtk.VBox()
        #vbox3.pack_start(gtk.Label('to be done'))

        # fill notebook
        self.notebook=gtk.Notebook()
        self.notebook.set_size_request(270,-1)
        self.notebook.append_page(vbox1,gtk.Label('display'))
        self.notebook.append_page(vbox2,gtk.Label('control'))
        self.notebook.append_page(logscroll,gtk.Label('log'))
        #self.notebook.append_page(vbox3,gtk.Label('info'))

        # centre hbox
        #~ hbox1 = gtk.HBox()
        #~ hbox1.pack_start(self.notebook,expand=False,fill=False)
        #~ hbox1.pack_start(self.widget,expand=True,fill=True)
        hbox1 = gtk.HPaned()
        hbox1.pack1(self.notebook,shrink=False)
        hbox1.pack2(self.widget)

        # main vbox
        vbox = gtk.VBox()
        window.add(vbox)
        vbox.pack_start(self.toolbar, False)
        vbox.pack_start(hbox1,True)
        vbox.pack_start(hbox_statusbars,False)

        self.set_focus(self.widget)

        self.show_all()

        # connect key press events

        self.connect('key-press-event', self.on_key_press_event)

    def on_key_press_event(self, widget, event):

        #### DEBUG
        #~ keyname = gtk.gdk.keyval_name(event.keyval)
        #~ print "Key %s (%d) was pressed" % (keyname, event.keyval)
        #~ if event.state & gtk.gdk.CONTROL_MASK:
            #~ print "Control was being held down"
        #~ if event.state & gtk.gdk.MOD1_MASK:
            #~ print "Alt was being held down"
        #~ if event.state & gtk.gdk.SHIFT_MASK:
            #~ print "Shift was being held down"

        ######
        if event.keyval == gtk.keysyms.space and not (event.state & gtk.gdk.CONTROL_MASK):
            self.on_run(widget)
            return True
        if event.keyval == gtk.keysyms.space and event.state & gtk.gdk.CONTROL_MASK:
            self.on_run_nsteps(widget)
            return True
        if event.keyval == gtk.keysyms.r:
            self.on_refresh(widget)
            return True
        if event.keyval == gtk.keysyms.e:
            self.on_export(widget)
            return True
        if event.keyval == gtk.keysyms.d:
            self.on_debug(widget)
            return True
        if event.keyval == gtk.keysyms.i:
            self.on_about(widget)
            return True
        if event.keyval == gtk.keysyms.f:
            if self.is_fullscreen == False:
                self.is_fullscreen = True
                self.fullscreen()
            else:
                self.is_fullscreen = False
                self.unfullscreen()
            return True
        if event.keyval == gtk.keysyms.g:
            self.notebook.set_current_page(1)
            self.entry_goto_task.grab_focus()
            return True
        if event.keyval == gtk.keysyms.c:# and event.state == gtk.gdk.CONTROL_MASK:
            self.on_connect(widget)
            return True
        if event.keyval == gtk.keysyms.z:
            self.widget.on_zoom_100(widget)
            return True
        if event.keyval == gtk.keysyms.Escape:
            self.widget.on_zoom_fit(widget)
            return True
        if not self.notebook.is_focus():
            if event.keyval == gtk.keysyms.Left:
                self.widget.x -= self.widget.POS_INCREMENT/self.widget.zoom_ratio
                self.widget.queue_draw()
                return True
            if event.keyval == gtk.keysyms.Right:
                self.widget.x += self.widget.POS_INCREMENT/self.widget.zoom_ratio
                self.widget.queue_draw()
                return True
            if event.keyval == gtk.keysyms.Up:
                self.widget.y -= self.widget.POS_INCREMENT/self.widget.zoom_ratio
                self.widget.queue_draw()
                return True
            if event.keyval == gtk.keysyms.Down:
                self.widget.y += self.widget.POS_INCREMENT/self.widget.zoom_ratio
                self.widget.queue_draw()
                return True
        if event.keyval == gtk.keysyms.Page_Down or event.keyval == gtk.keysyms.minus:
            self.widget.zoom_image(self.widget.zoom_ratio / self.widget.ZOOM_INCREMENT)
            self.widget.queue_draw()
            return True
        if event.keyval == gtk.keysyms.Page_Up or event.keyval == gtk.keysyms.plus:
            self.widget.zoom_image(self.widget.zoom_ratio * self.widget.ZOOM_INCREMENT)
            self.widget.queue_draw()
            return True
        if event.keyval == gtk.keysyms.p:
            self.on_settings(widget)
            return True
        if event.keyval == gtk.keysyms.q and event.state & gtk.gdk.CONTROL_MASK:
            self.on_quit(widget)
            return True

        return False


    def on_controlcheckbutton(self,checkbutton,data=None):
        if data == 'anyEvent':
            if checkbutton.get_active(): # is checked
                self.check_preruntask.set_sensitive(False)
                self.check_removetask.set_sensitive(False)
                self.check_waiton.set_sensitive(False)
                self.check_barrier.set_sensitive(False)
                if self.comm != None:
                    self.comm.send_request(0,'AYU_PAUSEONEVENT',
                                           [event_t.index('AYU_EVENT_NULL'),1])
            else: # is not checked
                self.check_preruntask.set_sensitive(True)
                self.check_removetask.set_sensitive(True)
                self.check_waiton.set_sensitive(True)
                self.check_barrier.set_sensitive(True)
                if self.comm != None:
                    self.comm.send_request(0,'AYU_PAUSEONEVENT',
                                           [event_t.index('AYU_EVENT_NULL'),0])
        if self.comm != None:
            if data == 'preruntask':
                if checkbutton.get_active():
                    self.comm.send_request(0,'AYU_PAUSEONEVENT',
                                           [event_t.index('AYU_PRERUNTASK'),1])
                else:
                    self.comm.send_request(0,'AYU_PAUSEONEVENT',
                                           [event_t.index('AYU_PRERUNTASK'),0])
            elif data == 'removetask':
                if checkbutton.get_active():
                    self.comm.send_request(0,'AYU_PAUSEONEVENT',
                                           [event_t.index('AYU_REMOVETASK'),1])
                else:
                    self.comm.send_request(0,'AYU_PAUSEONEVENT',
                                           [event_t.index('AYU_REMOVETASK'),0])
            elif data == 'waiton':
                if checkbutton.get_active():
                    self.comm.send_request(0,'AYU_PAUSEONEVENT',
                                           [event_t.index('AYU_WAITON'),1])
                else:
                    self.comm.send_request(0,'AYU_PAUSEONEVENT',
                                           [event_t.index('AYU_WAITON'),0])
            elif data == 'barrier':
                if checkbutton.get_active():
                    self.comm.send_request(0,'AYU_PAUSEONEVENT',
                                           [event_t.index('AYU_BARRIER'),1])
                else:
                    self.comm.send_request(0,'AYU_PAUSEONEVENT',
                                           [event_t.index('AYU_BARRIER'),0])
            elif data == 'preselecttask':
                if checkbutton.get_active():
                    self.comm.send_request(0,'AYU_BREAKPOINT',
                                           [event_t.index('AYU_PRESELECTTASK'),1])
                else:
                    self.comm.send_request(0,'AYU_BREAKPOINT',
                                           [event_t.index('AYU_PRESELECTTASK'),0])
            elif data == 'stopontask':
                if self.check_stopontask.get_active():
                    if self.label_stopontask.get_text() != '-none-':
                        taskId = int(self.label_stopontask.get_text())
                        self.comm.send_request(0,'AYU_PAUSEONTASK',
                                               [taskId,1])
                    else:
                        self.comm.send_request(0,'AYU_PAUSEONTASK',
                                               [0,0])
            # elif data == 'debugfunction':
            #     funcId = self.combo_debugfunction.get_active()-1
            #     if funcId >= 0:
            #         if self.check_debugfunction.get_active():
            #             #self.comm.send_request(0,'AYU_PAUSEONFUNCTION',
            #             #                       funcId,1)
            #             # FIXME: CN OMPSs fix as we get "Task 'functionname'" as task function names
            #             debugfunction_name = string.split(self.get_active_text(self.combo_debugfunction),"'")[1]
            #             self.start_gdb(self.executable, self.execut.pid,gdb_cmd=['break '+ debugfunction_name,'c'])
            #         else:
            #             pass
            #             #self.comm.send_request(0,'AYU_PAUSEONFUNCTION',
            #             #                       funcId,0)
#            elif data == 'combo_debugfunction':
#                 funcId = self.combo_debugfunction.get_active()-1
#                 if funcId >= 0:
#                     if self.check_debugfunction.get_active():
# #                    self.comm.send_request(0,'AYU_PAUSEONFUNCTION',
# #                                               funcId,1)
#                         # FIXME: CN OMPSs fix as we get "Task 'functionname'" as task function names
#                         debugfunction_name = string.split(self.get_active_text(self.combo_debugfunction),"'")[1]
#                         self.start_gdb(self.executable, self.execut.pid,gdb_cmd=['break '+ debugfunction_name,'c'])
#                     else:
#                         self.check_debugfunction.set_active(True)

    def on_nodecollist(self,widget,event):

        self.liststore_nodecol.clear()
#        iterat=self.liststore_nodecol.get_iter_first()
#        while self.liststore_nodecol.remove(iterat):
#            pass

        cols = self.treeview_nodecol.get_columns()
        t = self.get_active_text(self.combo_nodecollist)
        if t == 'function':
            self.widget.graphview.node_colours = t

            self.tvnc_col0.set_title('function')
            self.tvnc_col6.set_title('total')

            if self.tvnc_col1 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col1,1)
            if self.tvnc_col2 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col2,2)
            if self.tvnc_col3 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col3,3)
            if self.tvnc_col4 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col4,4)
#            if self.tvnc_col5 not in cols:
#                self.treeview_nodecol.insert_column(self.tvnc_col5,5)

            # fill function list
            for (fId,fName) in self.graph._funcName.iteritems():
                rgb=ayu_palette_rgb(fId,1.0)
                rgb=map(lambda a: int(255*a),rgb)
                color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                self.liststore_nodecol.append([fName, color_str,'0','0','0','0','0','0'])

            self.liststore_nodecol.append(['total', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'thread':
            self.widget.graphview.node_colours = t

            self.tvnc_col0.set_title('thread')
            self.tvnc_col6.set_title('total')

            if self.tvnc_col1 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col1)
            if self.tvnc_col2 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col2,1)
            if self.tvnc_col3 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col3,2)
            if self.tvnc_col4 not in cols:
                self.treeview_nodecol.insert_column(self.tvnc_col4,3)
#            if self.tvnc_col5 not in cols:
#                self.treeview_nodecol.insert_column(self.tvnc_col5,4)

            # fill thread list
            for thId in range(self.threads_dis+1):
                rgb=ayu_palette_rgb(thId,1.0)
                rgb=map(lambda a: int(255*a),rgb)
                color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                self.liststore_nodecol.append(['thread %d'%thId, color_str,'0','0','0','0','0','0'])
            self.liststore_nodecol.append(['total', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'task status':
            self.widget.graphview.node_colours = 'status'

            self.tvnc_col0.set_title('task status')
            self.tvnc_col6.set_title('#')

            if self.tvnc_col1 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col1)
            if self.tvnc_col2 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col2)
            if self.tvnc_col3 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col3)
            if self.tvnc_col4 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col4)
#            if self.tvnc_col5 in cols:
#                self.treeview_nodecol.remove_column(self.tvnc_col5)

            # fill status list
            self.liststore_nodecol.append(['not queued', '#ff0000','0','0','0','0','0','0'])
            self.liststore_nodecol.append(['queued', '#ffff00','0','0','0','0','0','0'])
            self.liststore_nodecol.append(['running', '#00ff00','0','0','0','0','0','0'])
            self.liststore_nodecol.append(['finished', '#ffffff','0','0','0','0','0','0'])
            self.liststore_nodecol.append(['failed', '#888888','0','0','0','0','0','0'])
            self.liststore_nodecol.append(['total', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'task duration':
            self.widget.graphview.node_colours = 'task_duration'
            self.tvnc_col0.set_title('task duration')
            self.tvnc_col6.set_title('cycles')

            if self.tvnc_col1 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col1)
            if self.tvnc_col2 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col2)
            if self.tvnc_col3 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col3)
            if self.tvnc_col4 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col4)
#            if self.tvnc_col5 in cols:
#                self.treeview_nodecol.remove_column(self.tvnc_col5)

            self.widget.graphview.get_dtminmax()
            # fill status list
            if self.widget.graphview.dt_min <= self.widget.graphview.dt_max:
                rgb=ayu_palette_rgb_g2r(0.001,math.log(self.widget.graphview.dt_max-self.widget.graphview.dt_min+1)+.001)
                rgb=map(lambda a: int(255*a),rgb)
                color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                self.liststore_nodecol.append(['min', color_str,'0','0','0','0','0',str(self.widget.graphview.dt_min)])
                for i in range (3,12):
                    i10 = 10**i
                    if i10 > self.widget.graphview.dt_min and i10 < self.widget.graphview.dt_max:
                        rgb=ayu_palette_rgb_g2r(math.log(i10-self.widget.graphview.dt_min+1)+.001,
                                                math.log(self.widget.graphview.dt_max-self.widget.graphview.dt_min+1)+.001)
                        rgb=map(lambda a: int(255*a),rgb)
                        color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                        self.liststore_nodecol.append(['', color_str,'0','0','0','0','0',str(i10)])

                rgb=ayu_palette_rgb_g2r(1,1)
                rgb=map(lambda a: int(255*a),rgb)
                color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                self.liststore_nodecol.append(['max', color_str,'0','0','0','0','0',str(self.widget.graphview.dt_max)])

        elif t == 'distance to marked':
            self.widget.graphview.node_colours = 'distance_to_marked'
            self.tvnc_col0.set_title('distance to marked')
            self.tvnc_col6.set_title('#')

            if self.tvnc_col1 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col1)
            if self.tvnc_col2 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col2)
            if self.tvnc_col3 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col3)
            if self.tvnc_col4 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col4)
#            if self.tvnc_col5 in cols:
#                self.treeview_nodecol.remove_column(self.tvnc_col5)

            self.widget.graphview.calc_all_distances(self.widget.graphview.marked_node)

            # fill colour list
            if self.widget.graphview.marked_node != None and self.widget.graphview.dist_max != 0:
                rgb=ayu_palette_rgb_g2r(1,self.widget.graphview.dist_max)
                rgb=map(lambda a: int(255*a),rgb)
                color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                self.liststore_nodecol.append(['min', color_str,'0','0','0','0','0','1'])
                for i in range (5,self.widget.graphview.dist_max,5):
                    print i
                    if i <= self.widget.graphview.dist_max:
                        rgb=ayu_palette_rgb_g2r(i,self.widget.graphview.dist_max)
                        rgb=map(lambda a: int(255*a),rgb)
                        color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                        self.liststore_nodecol.append(['', color_str,'0','0','0','0','0',str(i)])

                rgb=ayu_palette_rgb_g2r(1,1)
                rgb=map(lambda a: int(255*a),rgb)
                color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                self.liststore_nodecol.append(['max', color_str,'0','0','0','0','0',str(self.widget.graphview.dist_max)])

        elif t == 'nothing':
            self.widget.graphview.node_colours = 'nothing'
            self.liststore_nodecol.clear()

            self.tvnc_col0.set_title('no')
            self.tvnc_col6.set_title('colours')

            if self.tvnc_col1 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col1)
            if self.tvnc_col2 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col2)
            if self.tvnc_col3 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col3)
            if self.tvnc_col4 in cols:
                self.treeview_nodecol.remove_column(self.tvnc_col4)
#            if self.tvnc_col5 in cols:
#                self.treeview_nodecol.remove_column(self.tvnc_col5)
            else:
                self.liststore_nodecol.append(['no data','white','0','0','0','0','0','0'])

        self.update_ui()

    def on_margcollist(self,widget,event):

        self.liststore_margcol.clear()
#        iterat=self.liststore_margcol.get_iter_first()
#        while self.liststore_margcol.remove(iterat):
#            pass

        cols = self.treeview_margcol.get_columns()
        t = self.get_active_text(self.combo_margcollist)
        if t == 'function':
            self.widget.graphview.margins_mean = t

            self.tvmc_col0.set_title('function')
            self.tvmc_col6.set_title('total')

            if self.tvmc_col1 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col1,1)
            if self.tvmc_col2 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col2,2)
            if self.tvmc_col3 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col3,3)
            if self.tvmc_col4 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col4,4)
#            if self.tvmc_col5 not in cols:
#                self.treeview_margcol.insert_column(self.tvmc_col5,5)

            # fill function list
            for (fId,fName) in self.graph._funcName.iteritems(): # TODO: rough hack
                rgb=ayu_palette_rgb(fId,1.0)
                rgb=map(lambda a: int(255*a),rgb)
                color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                self.liststore_margcol.append([fName, color_str,'0','0','0','0','0','0'])

            self.liststore_margcol.append(['total', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'thread':
            self.widget.graphview.margins_mean = t

            self.tvmc_col0.set_title('thread')
            self.tvmc_col6.set_title('total')

            if self.tvmc_col1 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col1)
            if self.tvmc_col2 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col2,1)
            if self.tvmc_col3 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col3,2)
            if self.tvmc_col4 not in cols:
                self.treeview_margcol.insert_column(self.tvmc_col4,3)
#            if self.tvmc_col5 not in cols:
#                self.treeview_margcol.insert_column(self.tvmc_col5,4)

            # fill thread list
            for thId in range(self.threads_dis+1):
                rgb=ayu_palette_rgb(thId,1.0)
                rgb=map(lambda a: int(255*a),rgb)
                color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                self.liststore_margcol.append(['thread %d'%thId, color_str,'0','0','0','0','0','0'])
            self.liststore_margcol.append(['total', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'task status':
            self.widget.graphview.margins_mean = 'status'

            self.tvmc_col0.set_title('task status')
            self.tvmc_col6.set_title('#')

            if self.tvmc_col1 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col1)
            if self.tvmc_col2 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col2)
            if self.tvmc_col3 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col3)
            if self.tvmc_col4 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col4)
#            if self.tvmc_col5 in cols:
#                self.treeview_margcol.remove_column(self.tvmc_col5)

            # fill status list
            self.liststore_margcol.append(['not queued', 'red','0','0','0','0','0','0'])
            self.liststore_margcol.append(['queued', 'yellow','0','0','0','0','0','0'])
            self.liststore_margcol.append(['running', 'green','0','0','0','0','0','0'])
            self.liststore_margcol.append(['finished', 'white','0','0','0','0','0','0'])
            self.liststore_margcol.append(['failed', 'gray','0','0','0','0','0','0'])
            self.liststore_margcol.append(['total', 'white', '0', '0', '0', '0', '0', '0'])

        elif t == 'task duration':
            self.widget.graphview.margins_mean = 'task_duration'
            self.tvmc_col0.set_title('task duration')
            self.tvmc_col6.set_title('cycles')

            if self.tvmc_col1 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col1)
            if self.tvmc_col2 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col2)
            if self.tvmc_col3 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col3)
            if self.tvmc_col4 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col4)
#            if self.tvmc_col5 in cols:
#                self.treeview_margcol.remove_column(self.tvmc_col5)

            self.widget.graphview.get_dtminmax()
            # fill status list
            if self.widget.graphview.dt_min <= self.widget.graphview.dt_max:
                rgb=ayu_palette_rgb_g2r(0.001,math.log(self.widget.graphview.dt_max-self.widget.graphview.dt_min+1)+.001)
                rgb=map(lambda a: int(255*a),rgb)
                color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                self.liststore_margcol.append(['min', color_str,'0','0','0','0','0',str(self.widget.graphview.dt_min)])
                for i in range (3,12):
                    i10 = 10**i
                    if i10 > self.widget.graphview.dt_min and i10 < self.widget.graphview.dt_max:
                        rgb=ayu_palette_rgb_g2r(math.log(i10-self.widget.graphview.dt_min+1)+.001,
                                                math.log(self.widget.graphview.dt_max-self.widget.graphview.dt_min+1)+.001)
                        rgb=map(lambda a: int(255*a),rgb)
                        color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                        self.liststore_margcol.append(['', color_str,'0','0','0','0','0',str(i10)])

                rgb=ayu_palette_rgb_g2r(1,1)
                rgb=map(lambda a: int(255*a),rgb)
                color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                self.liststore_margcol.append(['max', color_str,'0','0','0','0','0',str(self.widget.graphview.dt_max)])
            else:
                self.liststore_margcol.append(['no data','white','0','0','0','0','0','0'])

        elif t == 'nothing':
            self.widget.graphview.margins_mean = 'nothing'

            self.tvmc_col0.set_title('no')
            self.tvmc_col6.set_title('margins')

            if self.tvmc_col1 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col1)
            if self.tvmc_col2 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col2)
            if self.tvmc_col3 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col3)
            if self.tvmc_col4 in cols:
                self.treeview_margcol.remove_column(self.tvmc_col4)
#            if self.tvmc_col5 in cols:
#                self.treeview_margcol.remove_column(self.tvmc_col5)

        self.update_ui()

    def on_shapelist(self,widget,event):

        self.liststore_shape.clear()
#        iterat=self.liststore_shape.get_iter_first()
#        while self.liststore_shape.remove(iterat):
#            pass

        cols = self.treeview_shape.get_columns()
        t = self.get_active_text(self.combo_shapelist)
        if t == 'function':
            self.widget.graphview.shapes_mean = 'function'

            self.tvns_col0.set_title('shape')
            self.tvns_col7.set_title('function')
            self.tvns_col6.set_title('total')

            if self.tvns_col1 not in cols:
                self.treeview_shape.insert_column(self.tvns_col1,2)
            if self.tvns_col2 not in cols:
                self.treeview_shape.insert_column(self.tvns_col2,3)
            if self.tvns_col3 not in cols:
                self.treeview_shape.insert_column(self.tvns_col3,4)
            if self.tvns_col4 not in cols:
                self.treeview_shape.insert_column(self.tvns_col4,5)
#            if self.tvns_col5 not in cols:
#                self.treeview_shape.insert_column(self.tvns_col5,6)

            # fill function list
            for (fId,fName) in self.graph._funcName.iteritems():
                if fId < len(self.pb_list)-1:
                    self.liststore_shape.append([self.pb_list[fId],
                                                 fName,
                                                 '0','0','0','0','0','0'])
                else:
                    self.liststore_shape.append([self.pb_list[len(self.pb_list)-1],
                                                 fName,
                                                 '0','0','0','0','0','0'])

            self.liststore_shape.append([None, 'total', '0', '0', '0', '0', '0', '0'])

        elif t == 'thread':
            self.widget.graphview.shapes_mean = 'thread'

            self.tvns_col0.set_title('shape')
            self.tvns_col7.set_title('thread')
            self.tvns_col6.set_title('total')

            if self.tvns_col1 in cols:
                self.treeview_shape.remove_column(self.tvns_col1)
            if self.tvns_col2 not in cols:
                self.treeview_shape.insert_column(self.tvns_col2,2)
            if self.tvns_col3 not in cols:
                self.treeview_shape.insert_column(self.tvns_col3,3)
            if self.tvns_col4 not in cols:
                self.treeview_shape.insert_column(self.tvns_col4,4)
#            if self.tvns_col5 not in cols:
#                self.treeview_shape.insert_column(self.tvns_col5,5)

            # fill thread list
            for thId in range(self.threads_dis):
                self.liststore_shape.append([self.pb_list[thId],'thread %d'%thId, '0','0','0','0','0','0'])

            self.liststore_shape.append([self.pb_list[len(self.shape_list)],'comm thread', '0','0','0','0','0','0'])

            self.liststore_shape.append([None, 'total', '0', '0', '0', '0', '0', '0'])

        elif t == 'nothing':
            self.widget.graphview.shapes_mean = 'nothing'

            self.tvns_col0.set_title('no')
            self.tvns_col6.set_title('shapes')
            self.tvns_col7.set_title('')

            if self.tvns_col1 in cols:
                self.treeview_shape.remove_column(self.tvns_col1)
            if self.tvns_col2 in cols:
                self.treeview_shape.remove_column(self.tvns_col2)
            if self.tvns_col3 in cols:
                self.treeview_shape.remove_column(self.tvns_col3)
            if self.tvns_col4 in cols:
                self.treeview_shape.remove_column(self.tvns_col4)
#            if self.tvns_col5 in cols:
#                self.treeview_shape.remove_column(self.tvns_col5)

        self.update_ui()

    def on_debugfunction(self,widget,event):
        funcId = self.combo_debugfunction.get_active()-1
        if funcId >= 0:
            debugfunction_name = self.get_active_text(self.combo_debugfunction)
            #debugfunction_name = string.split(self.get_active_text(self.combo_debugfunction),"'")[1]
            self.start_gdb(self.executable, self.execut.pid,gdb_cmd=['break '+ debugfunction_name,'c'])

    def on_clearmarked(self,widget,event):
        for n in self.graph:
            self.graph.highlight[n]=0
        self.widget.graphview.marked_node = None
        self.widget.queue_draw()


    def on_setnumthreads(self,widget,event,key=None):
        if self.comm != None:
            self.comm.send_request(0,'AYU_SETNUMTHREADS',
                [self.spin_numthreads.get_value_as_int()])

    def on_gototask(self,widget,event,key=None):
        taskId_text = self.entry_goto_task.get_text()
        if taskId_text.isdigit():
            taskId = int(self.entry_goto_task.get_text())
        else:
            return
        if taskId > self.graph.order():
            return
        self.widget.x = self.widget.graphview.layout[taskId][0]
        self.widget.y = -self.widget.graphview.layout[taskId][1]
        self.widget.animate_to(self.widget.x,self.widget.y)

    def on_debug(self,action):
        self.start_gdb(self.executable, self.execut.pid)

    def start_gdb(self,executable,pid,gdb_cmd=None):
        if gdb_cmd != None:
            # write commands for gdb to temporary file
            tmpfile=tempfile.mkstemp()
            os.write(tmpfile[0],'\n'.join(gdb_cmd)+'\n')
            os.close(tmpfile[0])

        # create gdb command line
        cmd = ['xterm','-geometry','80x40','-title',
                'gdb started by TEMANEJO',
                '-e','gdb','-tui',executable,
                str(pid)]
        if gdb_cmd != None:
            cmd += ['-x',tmpfile[1]]

        # execute gdb
        self.gdb=subprocess.Popen(cmd)#,stdin=subprocess.PIPE)

        # remove temporary file
        if gdb_cmd != None:
            time.sleep(2)
            os.unlink(tmpfile[1])

    def on_export(self,action):
        export_dlg = ExportDialog(self)
        response = export_dlg.run()
        if response == gtk.RESPONSE_OK:
            format = export_dlg.get_format()
            filename = export_dlg.get_filename()
            quality = export_dlg.get_quality()
            framerate = export_dlg.get_framerate()
            export_dlg.destroy()

            if format == 'text':
                self.export_graph_text(filename)
            elif format == 'png':
                self.export_graph_image(filename,format)
            elif format == 'jpeg':
                self.export_graph_image(filename,format,quality)
            elif format == 'dot':
                nx.write_dot(self.graph,filename)
            elif format == 'png_ser':
                self.export_graph_image(filename.replace('NNNNN','%05d'%0),'png')
                for i in range(1,self.graph.order()+1-self.status_counter_F): # get number of unfinished tasks
                    self.on_run('export')
                    self.export_graph_image(filename.replace('NNNNN','%05d'%i),'png')

            if format == 'gif':
                # save temporary png files:
                temp_filename_base = 'temanejo_TEMP' + time.strftime('_%Y%m%d-%H%M%S.png', time.localtime())
                self.export_graph_image(temp_filename_base+'-%05d'%0,'png')
                for i in range(1,self.graph.order()+1-self.status_counter_F): # get number of unfinished tasks
                    self.on_run('export')
                    self.export_graph_image(temp_filename_base+'-%05d'%i,'png')
                # convert to animation
                res = os.system('convert -delay ' + str(100/framerate) + ' ' + temp_filename_base + '* ' + filename)
                if res == 127:
                    print __progname__, 'ERROR: convert (ImageMagick) is not installed'
                elif res != 0:
                    print __progname__, 'ERROR: convert exited with error code', res
                else:
                    os.system('rm -f '+ temp_filename_base + '*')
            if format == 'avi':
                # save temporary jpeg files:
                temp_filename_base = 'temanejo_TEMP' + time.strftime('_%Y%m%d-%H%M%S.jpg', time.localtime())
                self.export_graph_image(temp_filename_base+'-%05d'%0,'jpeg',quality)
                for i in range(1,self.graph.order()+1-self.status_counter_F): # get number of unfinished tasks
                    self.on_run('export')
                    self.export_graph_image(temp_filename_base+'-%05d'%i,'jpeg',quality)
                # convert to animation
                res = os.system('mencoder "mf://' + temp_filename_base + '*" -mf fps='\
                    + str(framerate) + ' -mf type=jpeg -o ' + filename + ' -ovc lavc -lavcopts vcodec=mpeg4:vbitrate=1800')



                if res == 127:
                    print __progname__, 'ERROR: convert (ImageMagick) is not installed'
                elif res != 0:
                    print __progname__, 'ERROR: convert exited with error code', res
                else:
                    os.system('rm -f '+ temp_filename_base + '*')
        else:
            export_dlg.destroy()


    def export_graph_text(self,filename):
        if filename == None or filename == '':
            print __progname__, 'ERROR: invalid filename'
            return
            
        file_basename = filename.rpartition('.')[0]
        f = open(file_basename + '.nodes','w')
        f.write('#taskId\tfuncId\tfuncname\tduration\n')
            
        for n in self.graph.nodes():
            taskId = self.graph.get_taskId(n)
            funcId = self.graph.get_funcId(taskId)
            funcname = self.graph.get_funcname(funcId)
            if self.graph._status[n] != 3:
                dt = -1
            else:
                dt = self.graph.node[n]['ts_f']-self.graph.node[n]['ts_r']
            f.write('%d\t%d\t%s\t%d\n'%(taskId , funcId , funcname, dt))
            
        f.close

        f = open(file_basename + '.edges','w')
        f.write('#taskId\ttaskId\tmemaddr\n')
            
        for e in self.graph.edges():
            e_0 = e[0]
            e_1 = e[1]
            for ee in self.graph[e_0][e_1]:
                t = hex(self.graph[e_0][e_1][ee]['orig_addr'])
                f.write('%d\t%d\t%s\n'%(e_0 , e_1 , t))
        
        f.close

    def export_graph_image(self,filename,format,quality=None):
        rect = self.widget.get_allocation()
        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, rect.width, rect.height)
        pixbuf.get_from_drawable(self.widget.get_snapshot(),
                                    gtk.gdk.colormap_get_system(),
                                    0, 0, 0, 0,
                                    rect.width, rect.height)
        #print "writing", format, "to", filename
        if quality != None:
            pixbuf.save(filename,format, {"quality":str(quality)})
        else:
            pixbuf.save(filename,format)


    def cleanup(self):
        # graph
        self.graph.clear()

        # last event and taskId for status bar
        self.last_event='AYU_EVENT_NULL'
        self.last_check_event='AYU_EVENT_NULL'
        self.last_taskId=-1

        # widget
        self.widget.first_drawing_event=True
        self.widget.graphview.clear()

        # communticator thread
        if self.comm != None:
            self.comm.terminate()
            self.comm=None

        # thread for execution of the application
        if self.execut != None:
            os.system('kill ' + str(self.execut.pid))
            self.execut=None

        # number of registered functions
        self.func_count=0

        # barrier counter
        self.barrier_count = 0

        # counters
        self.func_counter    ={} # total count per function
        self.func_counter_NQ ={} # not queued
        self.func_counter_Q  ={} # queued
        self.func_counter_R  ={} # running
        self.func_counter_F  ={} # finished
#        self.func_counter_X  ={} # failed

        self.thread_counter    ={} # total
#        self.thread_counter_NQ =0 # not queued
        self.thread_counter_Q  ={} # queued
        self.thread_counter_R  ={} # running
        self.thread_counter_F  ={} # finished
#        self.thread_counter_X  ={} # failed

        self.status_counter_NQ =0 # not queued
        self.status_counter_Q  =0 # queued
        self.status_counter_R  =0 # running
        self.status_counter_F  =0 # finished
#        self.status_counter_X  =0 # failed

        # node colour list
        self.liststore_nodecol.clear()
        tmp=['function','thread','status','task_duration','nothing']
        self.combo_nodecollist.set_active(-1)
        self.combo_nodecollist.set_active(tmp.index(self.widget.graphview.node_colours))

        # margin colour list
        self.liststore_margcol.clear()
        tmp=['status','function','thread','task_duration','nothing']
        self.combo_margcollist.set_active(-1)
        self.combo_margcollist.set_active(tmp.index(self.widget.graphview.margins_mean))

        # node shape list
        self.liststore_shape.clear()
        tmp=['thread','function','nothing']
        self.combo_shapelist.set_active(-1)
        self.combo_shapelist.set_active(tmp.index(self.widget.graphview.shapes_mean))

        # checkboxes etc.
        self.check_preselecttask.set_active(False)
        self.check_anyEvent.set_active(False)
        self.check_preruntask.set_active(True)
        self.check_removetask.set_active(False)
        self.check_waiton.set_active(False)
        self.check_barrier.set_active(False)
        self.check_stopontask.set_active(False)
#        self.check_debugfunction.set_active(False)
        self.label_stopontask.set_text('-none-')

        listmodel = self.combo_debugfunction.get_model()
        iterat = listmodel.get_iter_first()
        while listmodel.remove(iterat):
            pass
        listmodel.append(['-none-'])
        self.combo_debugfunction.set_active(0)

        self.widget.queue_draw()

    def get_active_text(self,combobox):
        model = combobox.get_model()
        active = combobox.get_active()
        if active < 0:
            return None
        return model[active][0]

    def on_connect(self, action):
        # connection Dialog ----------------------
        connect_dlg = ConnectDialog(self)

        response = connect_dlg.run()
        if response == gtk.RESPONSE_ACCEPT:
            # cleanup --------------------------------
            self.cleanup()

            # get dialog input
            if connect_dlg.get_local_remote() == 0: # local
                self.executable = connect_dlg.get_exec()
                self.executable_args = connect_dlg.get_exec_args()
#                print self.executable_args
                self.library = connect_dlg.get_lib()
                configuration = connect_dlg.get_cfg()
                ayu_host = 'localhost'
                ayu_port = connect_dlg.get_port()
                cpus = connect_dlg.get_cpus()
                if VERBOSE > 0:
                    print __progname__, ': executable:           ', self.executable
                    print __progname__, ': executable arguments: ', self.executable_args
                    print __progname__, ': AYUDAME library:      ', self.library
                    print __progname__, ': SMPSS configuration:  ', configuration
                    print __progname__, ': hostname:             ', ayu_host
                    print __progname__, ': port:                 ', ayu_port
                    print __progname__, ': number of cpus:       ', cpus

            else: # remote
                self.executable = None
                self.executable_args = ''
                self.library = None
                configuration = None
                ayu_host = connect_dlg.get_host()
                ayu_port = connect_dlg.get_port()
                cpus = None
                if VERBOSE > 0:
                    print __progname__, ': hostname:            ', ayu_host
                    print __progname__, ': port:                ', ayu_port

            connect_dlg.destroy()

            # initialise thread counters
            if cpus != None:
                self.threads_dis = cpus
            else:
                self.threads_dis = THREADS_DISPLAY
            self.spin_numthreads.set_value(self.threads_dis)

            # set comm thread id
            self.widget.graphview.commthreadId = self.threads_dis

            for i in range(self.threads_dis+1):
                self.thread_counter[i]   =0
                self.thread_counter_Q[i] =0
                self.thread_counter_R[i] =0
                self.thread_counter_F[i] =0
#                self.thread_counter_X[i] =0
            self.thread_counter_Q[-1] =0
            self.thread_counter[-1] =0

	        # node colour list
            self.liststore_nodecol.clear()
            tmp=['function','thread','status','task_duration','nothing']
            self.combo_nodecollist.set_active(-1)
            self.combo_nodecollist.set_active(tmp.index(self.widget.graphview.node_colours))

            # margin colour list
            self.liststore_margcol.clear()
            tmp=['status','function','thread','task_duration','nothing']
            self.combo_margcollist.set_active(-1)
            self.combo_margcollist.set_active(tmp.index(self.widget.graphview.margins_mean))

            # node shape list
            self.liststore_shape.clear()
            tmp=['thread','function','nothing']
            self.combo_shapelist.set_active(-1)
            self.combo_shapelist.set_active(tmp.index(self.widget.graphview.shapes_mean))

            # -------------

        else:
            connect_dlg.destroy()
            return

        # ----------------------------------------
        # check response and run executable -------------------------
        if self.executable != None:
            if self.executable == '' or not os.path.isfile(self.executable):
                if VERBOSE > -1:
                    print __progname__ + ': Application not specified. Waiting for remote connection.'
#                return

            else:
                if self.library == None:
                    md = gtk.MessageDialog(self,
                                           gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING,
                                           gtk.BUTTONS_CLOSE, 'AYUDAME library not specified.\nPlease make sure libayudame is linked to the program.')
                    md.run()
                    md.destroy()
                    if VERBOSE > -1:
                        print __progname__ + ': AYUDAME library not specified.'
                    #return

                if '/' not in self.executable:
                    self.executable = './' + self.executable

                # ayudame specific environment variables

                env_dict = os.environ
                env_dict['AYU_PORT']        = str(ayu_port)
                # StarSs specific environment variables
                env_dict['CSS_NUM_CPUS']    = str(cpus)
                if configuration != None:
                    env_dict['CSS_CONFIG_FILE'] = configuration
                env_dict['LD_PRELOAD']      = self.library
                # Nanos++ specific environment variables
                # TODO: There are a lot more special vars like NX_SCHEDULE, ... to handle
                env_dict['NX_ARGS']         = os.getenv( 'NX_ARGS', '--instrumentation=ayudame' )
                env_dict['NX_PES']          = str(cpus)

                exec_cmd = [self.executable]
                exec_cmd += shlex.split(self.executable_args)
                try:
                    self.execut=subprocess.Popen(exec_cmd,env=env_dict)
                except OSError, e:
                    if VERBOSE > -1:
                        print __progname__, ": OSError:", e
                    return
                end=self.logbuffer.get_end_iter()
                self.logbuffer.insert(end,'\nExecuting ' + self.exec_cmd + ' ' +
                                          ' with PID '+ str(self.execut.pid))

                if VERBOSE > 0:
                    print __progname__, ": executing '" + string.join(exec_cmd) + "' with PID", self.execut.pid
                time.sleep(.5)

        # ----------------------------------------


        # set up client socket -------------------
        if VERBOSE > 0:
            print __progname__, ': creating socket ...'
        try:
            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        except socket.error, msg:
            self.sock = None
            if VERBOSE > -1:
                print __progname__, ': Error creating socket: ', msg
            return

        connect_count=0
        while(1):
            try:
                self.sock.connect((ayu_host, ayu_port))
            except socket.error, msg:
                if VERBOSE > 0:
                    print __progname__, ': Error conecting to server: ', ayu_host, ' port ', ayu_port
                    print __progname__, msg
                    print __progname__, ': Trying again...'
                time.sleep(.5)
                connect_count+=1
                if connect_count>3:
                    if VERBOSE > -1:
                        print __progname__, ': Connection to server could not be established.'
                    self.sock.close();
                    self.sock=None
                    return
            else:
                if VERBOSE > 0:
                    print __progname__, ': Connection to server established.'
                break
        # ----------------------------------------

        # launch communicater thread

        self.comm=Communicator(self.sock,self)
        self.comm.deamon=True
        self.comm.start()
        # ----------------------------------------


    def on_refresh(self, action):
        self.widget.graphview.is_invalid = True
        self.update_ui('all')

    def on_run(self, action):
        if self.comm == None:
            return
        self.actiongroup.get_action('Run_fast').set_sensitive(True)
        self.comm.send_request(0,'AYU_STEP',1)
        time.sleep(.1)
        self.update_ui('all')

    def on_run_nsteps(self, action):
        if self.comm == None:
            return
        self.comm.send_request(0,'AYU_STEP',self.n_steps)
        time.sleep(.1*math.sqrt(self.n_steps))
        self.update_ui('all')

    def on_about(self, action):
        dlg=AboutDialog()
        dlg.run()
        dlg.destroy()

    def on_settings(self, action):
        dlg=SettingsDialog(self)
        response=dlg.run()
        time.sleep(.2)
        if response == gtk.RESPONSE_OK or \
                response == gtk.RESPONSE_APPLY:
            self.widget.autozoom       = dlg.get_check_autozoom()
            self.widget.graphview.show_markers     = dlg.get_check_markers()
            self.widget.graphview.addr_key         = dlg.get_edge_colours()
            self.widget.graphview.draw_edge_labels = dlg.get_edge_labels()
            self.widget.graphview.margin_width     = dlg.get_spin_margin_width()
            self.widget.graphview.edge_width       = dlg.get_spin_edge_width()
            self.widget.graphview.node_size        = dlg.get_spin_node_size()
            if self.widget.graphview.layout_direction != dlg.get_graph_direction():
                self.widget.graphview.layout_direction = dlg.get_graph_direction()
                self.widget.zoom_once = True
                self.widget.graphview.is_invalid = True
            if self.n_steps != dlg.get_spin_nsteps():
                self.n_steps = dlg.get_spin_nsteps()
                toolbutton = self.uimanager.get_widget('/ToolBar/Run_fast')
                toolbutton.set_label('Run '+str(self.n_steps)+' tasks')
            if self.widget.graphview.use_layout  != dlg.get_graph_layout():
                self.widget.graphview.use_layout = dlg.get_graph_layout()
                self.widget.zoom_once = True
                self.widget.graphview.is_invalid = True
            if self.widget.graphview.global_tasks_in_first_row  != dlg.get_global_tasks_first_row():
                self.widget.graphview.global_tasks_in_first_row = dlg.get_global_tasks_first_row()
                self.widget.zoom_once = True
                self.widget.graphview.is_invalid = True

            self.widget.queue_draw()

        dlg.destroy()

    # def on_help(self, action):
    #     dlg=HelpDialog()
    #     dlg.run()
    #     dlg.destroy()

    def on_quit(self,action,data=None):
        if (VERBOSE > 0):
            print __progname__, ': Stopping communication thread.'
        if self.comm != None:
            self.comm.terminate()
            #self.sock.shutdown(socket.SHUT_RDWR)
            self.sock.close()

        if (VERBOSE > 0):
            print __progname__, ': Stopping application thread.'
        if self.execut != None:
#            self.execut.send_signal(signal.SIGTERM) # for python ver. >= 2.6
            os.system('kill ' + str(self.execut.pid))

        if (VERBOSE > -1):
            print '#',__progname__, ': ### Exiting ###'
        gtk.main_quit()

        return False

    def push_statusbar(self,msg=None):
        if msg == None:
            msg='last event: '  + self.last_event + ' for taskId ' + str(self.last_taskId)

        self.statusbar.pop(self.statusbar_lastmsg_cid)
        self.statusbar_lastmsg_id=self.statusbar.push(self.statusbar_lastmsg_cid,msg)

        self.statusbar2.pop(self.statusbar2_lastmsg_cid)
        self.statusbar2_lastmsg_id=self.statusbar2.push(
            self.statusbar2_lastmsg_cid,
            '%d nodes, %d edges, %d addresses, %d original addresses'%(self.graph.order(),
                                                                       self.graph.size(),
                                                                       len(self.graph.addresses),
                                                                       len(self.graph.orig_addresses)))


    def highlight_checkbox(self,event=None):
        if event == None:
            event=self.last_check_event
        # all to default
        self.eventbox_preruntask.modify_style(self.check_default_style)
        self.eventbox_removetask.modify_style(self.check_default_style)
        self.eventbox_waiton.modify_style(self.check_default_style)
        self.eventbox_barrier.modify_style(self.check_default_style)

        if event == 'AYU_PRERUNTASK' :
            self.eventbox_preruntask.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(65535,0,0));
        elif event == 'AYU_REMOVETASK' :
            self.eventbox_removetask.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(65535,0,0));
        elif event == 'AYU_WAITON' :
            self.eventbox_waiton.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(65535,0,0));
        elif event == 'AYU_BARRIER' :
            self.eventbox_barrier.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(65535,0,0));

        #self.eventbox_preruntask.queue_draw()
        #self.eventbox_removetask.queue_draw()
        #self.eventbox_waiton.queue_draw()
        #self.eventbox_barrier.queue_draw()

    def update_ui(self,widget='all'):
        # widget is one of 'all', 'log', 'statusbar', 'counters', 'graphwidget'

        if self.comm == None or self.comm.working or self.graph.order() == 0:
            return
        if widget == 'log' or widget == 'all':
            text=self.prelog
            self.prelog=''
            end = self.logbuffer.get_end_iter()
            self.logbuffer.insert(end, text)
            self.logview.scroll_mark_onscreen(self.logbuffer.get_insert())

#            self.logview.queue_draw()

#        if widget == 'checkboxes' or widget == 'all':
            # checkboxes
#            self.highlight_checkbox()
        if widget == 'statusbar' or widget == 'all':
            # status bar
            self.push_statusbar()

        if widget == 'counters' or widget == 'all':

            # node colour counters
            if self.widget.graphview.node_colours == 'function':
                # function counter table
                i=0
                for (fId,count) in self.func_counter.iteritems():
                    ls_iter=self.liststore_nodecol.get_iter(i)
                    i+=1
                    self.liststore_nodecol.set_value(ls_iter,2,str(self.func_counter_NQ[fId]))
                    self.liststore_nodecol.set_value(ls_iter,3,str(self.func_counter_Q[fId]))
                    self.liststore_nodecol.set_value(ls_iter,4,str(self.func_counter_R[fId]))
                    self.liststore_nodecol.set_value(ls_iter,5,str(self.func_counter_F[fId]))
#                    self.liststore_nodecol.set_value(ls_iter,6,str(self.func_counter_X[fId]))
                    self.liststore_nodecol.set_value(ls_iter,7,str(count))
                ls_iter=self.liststore_nodecol.get_iter(len(self.func_counter))
                self.liststore_nodecol.set_value(ls_iter,2,str(sum(self.func_counter_NQ.values())))
                self.liststore_nodecol.set_value(ls_iter,3,str(sum(self.func_counter_Q.values())))
                self.liststore_nodecol.set_value(ls_iter,4,str(sum(self.func_counter_R.values())))
                self.liststore_nodecol.set_value(ls_iter,5,str(sum(self.func_counter_F.values())))
#                self.liststore_nodecol.set_value(ls_iter,6,str(sum(self.func_counter_X.values())))
                self.liststore_nodecol.set_value(ls_iter,7,str(sum(self.func_counter.values())))

            elif self.widget.graphview.node_colours == 'thread':
                for thId in range(self.threads_dis+1):
                    ls_iter=self.liststore_nodecol.get_iter(thId)
                    self.liststore_nodecol.set_value(ls_iter,3,str(self.thread_counter_Q[thId]))
                    self.liststore_nodecol.set_value(ls_iter,4,str(self.thread_counter_R[thId]))
                    self.liststore_nodecol.set_value(ls_iter,5,str(self.thread_counter_F[thId]))
#                    self.liststore_nodecol.set_value(ls_iter,6,str(self.thread_counter_X[thId]))
                    self.liststore_nodecol.set_value(ls_iter,7,str(self.thread_counter[thId]))
                ls_iter=self.liststore_nodecol.get_iter(self.threads_dis+1)
                self.liststore_nodecol.set_value(ls_iter,3,str(sum(self.thread_counter_Q.values())))
                self.liststore_nodecol.set_value(ls_iter,4,str(sum(self.thread_counter_R.values())))
                self.liststore_nodecol.set_value(ls_iter,5,str(sum(self.thread_counter_F.values())))
#                self.liststore_nodecol.set_value(ls_iter,6,str(sum(self.thread_counter_X.values())))
                self.liststore_nodecol.set_value(ls_iter,7,str(sum(self.thread_counter.values())))

            elif self.widget.graphview.node_colours == 'status':
                self.liststore_nodecol.set_value(self.liststore_nodecol.get_iter(0),7,str(self.status_counter_NQ))
                self.liststore_nodecol.set_value(self.liststore_nodecol.get_iter(1),7,str(self.status_counter_Q))
                self.liststore_nodecol.set_value(self.liststore_nodecol.get_iter(2),7,str(self.status_counter_R))
                self.liststore_nodecol.set_value(self.liststore_nodecol.get_iter(3),7,str(self.status_counter_F))
#                self.liststore_nodecol.set_value(self.liststore_nodecol.get_iter(4),7,str(self.status_counter_X))
                self.liststore_nodecol.set_value(self.liststore_nodecol.get_iter(5),7,str(self.status_counter_NQ+
                                                              self.status_counter_Q+
                                                              self.status_counter_R+
                                                              self.status_counter_F))
#                                                              +self.status_counter_X))

            elif self.widget.graphview.node_colours == 'task_duration':
                self.widget.graphview.get_dtminmax()
                # fill colour list
                if self.widget.graphview.dt_min <= self.widget.graphview.dt_max:
                    self.liststore_nodecol.clear()
                    rgb=ayu_palette_rgb_g2r(0.001,math.log(self.widget.graphview.dt_max-self.widget.graphview.dt_min+1)+.001)
                    rgb=map(lambda a: int(255*a),rgb)
                    color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                    self.liststore_nodecol.append(['min', color_str,'0','0','0','0','0',str(self.widget.graphview.dt_min)])
                    for i in range (3,12):
                        i10 = 10**i
                        if i10 > self.widget.graphview.dt_min and i10 < self.widget.graphview.dt_max:
                            rgb=ayu_palette_rgb_g2r(math.log(i10-self.widget.graphview.dt_min+1)+.001,
                                                    math.log(self.widget.graphview.dt_max-self.widget.graphview.dt_min+1)+.001)
                            rgb=map(lambda a: int(255*a),rgb)
                            color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                            self.liststore_nodecol.append(['', color_str,'0','0','0','0','0',str(i10)])

                    rgb=ayu_palette_rgb_g2r(1,1)
                    rgb=map(lambda a: int(255*a),rgb)
                    color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                    self.liststore_nodecol.append(['max', color_str,'0','0','0','0','0',str(self.widget.graphview.dt_max)])
                else:
                    pass
                    #self.liststore_nodecol.append(['no data','white','0','0','0','0','0'])

            elif self.widget.graphview.node_colours == 'distance_to_marked':
                self.widget.graphview.calc_all_distances(self.widget.graphview.marked_node)

                # fill status list
                if self.widget.graphview.marked_node != None and self.widget.graphview.dist_max != 0:
                    self.liststore_nodecol.clear()
                    rgb=ayu_palette_rgb_g2r(1,self.widget.graphview.dist_max)
                    rgb=map(lambda a: int(255*a),rgb)
                    color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                    self.liststore_nodecol.append(['min', color_str,'0','0','0','0','0','1'])
                    for i in range (5,self.widget.graphview.dist_max,5):
                        if i <= self.widget.graphview.dist_max:
                            rgb=ayu_palette_rgb_g2r(i,self.widget.graphview.dist_max)
                            rgb=map(lambda a: int(255*a),rgb)
                            color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                            self.liststore_nodecol.append(['', color_str,'0','0','0','0','0',str(i)])

                    rgb=ayu_palette_rgb_g2r(1,1)
                    rgb=map(lambda a: int(255*a),rgb)
                    color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                    self.liststore_nodecol.append(['max', color_str,'0','0','0','0','0',str(self.widget.graphview.dist_max)])

            # margin colour counters
            if self.widget.graphview.margins_mean == 'function':
                # function counter table
                i=0
                for (fId,count) in self.func_counter.iteritems():
                    ls_iter=self.liststore_margcol.get_iter(i)
                    i+=1
                    self.liststore_margcol.set_value(ls_iter,2,str(self.func_counter_NQ[fId]))
                    self.liststore_margcol.set_value(ls_iter,3,str(self.func_counter_Q[fId]))
                    self.liststore_margcol.set_value(ls_iter,4,str(self.func_counter_R[fId]))
                    self.liststore_margcol.set_value(ls_iter,5,str(self.func_counter_F[fId]))
#                    self.liststore_margcol.set_value(ls_iter,6,str(self.func_counter_X[fId]))
                    self.liststore_margcol.set_value(ls_iter,7,str(count))
                ls_iter=self.liststore_margcol.get_iter(len(self.func_counter))
                self.liststore_margcol.set_value(ls_iter,2,str(sum(self.func_counter_NQ.values())))
                self.liststore_margcol.set_value(ls_iter,3,str(sum(self.func_counter_Q.values())))
                self.liststore_margcol.set_value(ls_iter,4,str(sum(self.func_counter_R.values())))
                self.liststore_margcol.set_value(ls_iter,5,str(sum(self.func_counter_F.values())))
#                self.liststore_margcol.set_value(ls_iter,6,str(sum(self.func_counter_X.values())))
                self.liststore_margcol.set_value(ls_iter,7,str(sum(self.func_counter.values())))

            elif self.widget.graphview.margins_mean == 'thread':
                for thId in range(self.threads_dis+1):
                    ls_iter=self.liststore_margcol.get_iter(thId)
                    self.liststore_margcol.set_value(ls_iter,3,str(self.thread_counter_Q[thId]))
                    self.liststore_margcol.set_value(ls_iter,4,str(self.thread_counter_R[thId]))
                    self.liststore_margcol.set_value(ls_iter,5,str(self.thread_counter_F[thId]))
#                    self.liststore_margcol.set_value(ls_iter,6,str(self.thread_counter_X[thId]))
                    self.liststore_margcol.set_value(ls_iter,7,str(self.thread_counter[thId]))
                ls_iter=self.liststore_margcol.get_iter(self.threads_dis+1)
                self.liststore_margcol.set_value(ls_iter,3,str(sum(self.thread_counter_Q.values())))
                self.liststore_margcol.set_value(ls_iter,4,str(sum(self.thread_counter_R.values())))
                self.liststore_margcol.set_value(ls_iter,5,str(sum(self.thread_counter_F.values())))
#                self.liststore_margcol.set_value(ls_iter,6,str(sum(self.thread_counter_X.values())))
                self.liststore_margcol.set_value(ls_iter,7,str(sum(self.thread_counter.values())))

            elif self.widget.graphview.margins_mean == 'status':
                self.liststore_margcol.set_value(self.liststore_margcol.get_iter(0),7,str(self.status_counter_NQ))
                self.liststore_margcol.set_value(self.liststore_margcol.get_iter(1),7,str(self.status_counter_Q))
                self.liststore_margcol.set_value(self.liststore_margcol.get_iter(2),7,str(self.status_counter_R))
                self.liststore_margcol.set_value(self.liststore_margcol.get_iter(3),7,str(self.status_counter_F))
#                self.liststore_margcol.set_value(self.liststore_margcol.get_iter(4),7,str(self.status_counter_X))
                self.liststore_margcol.set_value(self.liststore_margcol.get_iter(5),7,str(self.status_counter_NQ+
                                                              self.status_counter_Q+
                                                              self.status_counter_R+
                                                              self.status_counter_F))
#                                                              +self.status_counter_X))

            elif self.widget.graphview.margins_mean == 'task_duration':
                self.widget.graphview.get_dtminmax()
                # fill status list
                if self.widget.graphview.dt_min <= self.widget.graphview.dt_max:
                    self.liststore_margcol.clear()
                    rgb=ayu_palette_rgb_g2r(0.001,math.log(self.widget.graphview.dt_max-self.widget.graphview.dt_min+1)+.001)
                    rgb=map(lambda a: int(255*a),rgb)
                    color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                    self.liststore_margcol.append(['min', color_str,'0','0','0','0','0',str(self.graph.dt_min)])
                    for i in range (3,12):
                        i10 = 10**i
                        if i10 > self.widget.graphview.dt_min and i10 < self.widget.graphview.dt_max:
                            rgb=ayu_palette_rgb_g2r(math.log(i10-self.widget.graphview.dt_min+1)+.001,
                                                    math.log(self.widget.graphview.dt_max-self.widget.graphview.dt_min+1)+.001)
                            rgb=map(lambda a: int(255*a),rgb)
                            color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                            self.liststore_margcol.append(['', color_str,'0','0','0','0','0',str(i10)])

                    rgb=ayu_palette_rgb_g2r(1,1)
                    rgb=map(lambda a: int(255*a),rgb)
                    color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
                    self.liststore_margcol.append(['max', color_str,'0','0','0','0','0',str(self.widget.graphview.dt_max)])
                else:
                    pass
                    #self.liststore_margcol.append(['no data','white','0','0','0','0','0'])

            # node shape counters
            if self.widget.graphview.shapes_mean == 'function':
                # function counter table
                i=0
                for (fId,count) in self.func_counter.iteritems():
                    ls_iter=self.liststore_shape.get_iter(i)
                    i+=1
                    self.liststore_shape.set_value(ls_iter,2,str(self.func_counter_NQ[fId]))
                    self.liststore_shape.set_value(ls_iter,3,str(self.func_counter_Q[fId]))
                    self.liststore_shape.set_value(ls_iter,4,str(self.func_counter_R[fId]))
                    self.liststore_shape.set_value(ls_iter,5,str(self.func_counter_F[fId]))
#                    self.liststore_shape.set_value(ls_iter,6,str(self.func_counter_X[fId]))
                    self.liststore_shape.set_value(ls_iter,7,str(count))
                ls_iter=self.liststore_shape.get_iter(len(self.func_counter))
                self.liststore_shape.set_value(ls_iter,2,str(sum(self.func_counter_NQ.values())))
                self.liststore_shape.set_value(ls_iter,3,str(sum(self.func_counter_Q.values())))
                self.liststore_shape.set_value(ls_iter,4,str(sum(self.func_counter_R.values())))
                self.liststore_shape.set_value(ls_iter,5,str(sum(self.func_counter_F.values())))
#                self.liststore_shape.set_value(ls_iter,6,str(sum(self.func_counter_X.values())))
                self.liststore_shape.set_value(ls_iter,7,str(sum(self.func_counter.values())))

            elif self.widget.graphview.shapes_mean == 'thread':
                for thId in range(self.threads_dis+1):
                    ls_iter=self.liststore_shape.get_iter(thId)
#                    ls_iter=self.liststore_shape.get_iter(thId)
                    self.liststore_shape.set_value(ls_iter,3,str(self.thread_counter_Q[thId]))
                    self.liststore_shape.set_value(ls_iter,4,str(self.thread_counter_R[thId]))
                    self.liststore_shape.set_value(ls_iter,5,str(self.thread_counter_F[thId]))
#                    self.liststore_shape.set_value(ls_iter,6,str(self.thread_counter_X[thId]))
                    self.liststore_shape.set_value(ls_iter,7,str(self.thread_counter[thId]))
                ls_iter=self.liststore_shape.get_iter(self.threads_dis+1)
                self.liststore_shape.set_value(ls_iter,3,str(sum(self.thread_counter_Q.values())))
                self.liststore_shape.set_value(ls_iter,4,str(sum(self.thread_counter_R.values())))
                self.liststore_shape.set_value(ls_iter,5,str(sum(self.thread_counter_F.values())))
#                self.liststore_shape.set_value(ls_iter,6,str(sum(self.thread_counter_X.values())))
                self.liststore_shape.set_value(ls_iter,7,str(sum(self.thread_counter.values())))


        if widget == 'graphwidget' or widget == 'all':
        #if self.widget.autozoom or self.widget.zoom_once:
        #    self.widget.zoom_to_fit()
        #else:
        #    # draw!
            self.widget.queue_draw()

    def register_function(self,fId,fName):
        self.graph.set_funcname(fId,fName)

        # fill function list
        if self.widget.graphview.node_colours == 'function':

            rgb=ayu_palette_rgb(fId,1.0)
            rgb=map(lambda a: int(255*a),rgb)
            color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
            self.liststore_nodecol.insert(len(self.graph._funcName)-1,[fName, color_str,'0','0','0','0','0','0'])

        if self.widget.graphview.margins_mean == 'function':
            rgb=ayu_palette_rgb(fId,1.0)
            rgb=map(lambda a: int(255*a),rgb)
            color_str = '#%02x%02x%02x' % (rgb[0],rgb[1],rgb[2])
            self.liststore_margcol.insert(len(self.graph._funcName)-1,[fName, color_str,'0','0','0','0','0','0'])

        if self.widget.graphview.shapes_mean == 'function':
            # TODO!
            # check for number_of_functions > len(self.shape_list)
            self.liststore_shape.insert(len(self.graph._funcName)-1,[self.pb_list[fId],fName,'0','0','0','0','0','0'])

        # number of registered functions
        self.func_count+=1
        # initiate counters
        self.func_counter[fId]=0
        self.func_counter_NQ[fId]=0
        self.func_counter_Q[fId]=0
        self.func_counter_R[fId]=0
        self.func_counter_F[fId]=0
#        self.func_counter_X[fId]=0
        # fill combo list
        self.combo_debugfunction.append_text(fName)

#******

#****c* TEMANEJO/ConnectDialog
# NAME
#   ConnectDialog
#   - dialog for connecting to an AYUDAME server
# DESCRIPTION
#   The class implements a dialog where the user can choose an executable,
#   the location of the libayudame.so, a configuration file, port number
#   and number of cores. On pressing the connect button the executable will be
#   executed and pause when reaching css_init.
# DERIVED FROM
#   gtk.Dialog
# METHODS
#   __init__
#   callback_choose_cfg
#   callback_choose_exec
#   callback_choose_lib
#   get_cfg
#   get_cpus
#   get_exec
#   get_exec_args
#   get_host
#   get_lib
#   get_local_remote
#   get_port
# SOURCE

class ConnectDialog(gtk.Dialog):
    def __init__(self,parent):
        gtk.Dialog.__init__(self,title='Connect to AYUDAME...', parent=self,
                            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            buttons= (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                      gtk.STOCK_CONNECT, gtk.RESPONSE_ACCEPT))

        self.set_default_response(gtk.RESPONSE_ACCEPT)

        # notebook tab for local connection
        vbox0=gtk.VBox()

        frame = gtk.Frame('Specify the executable:')
        self.entry_exec = gtk.Entry(max=0)
        if parent.executable == None or len(parent.executable) == 0:
            self.entry_exec.set_text('#enter executable here#')
        else:
            self.entry_exec.set_text(parent.executable)
        self.entry_exec.set_width_chars(64)
        self.entry_exec.set_activates_default(True)
        button = gtk.Button(stock=gtk.STOCK_OPEN)
        button.connect('clicked', self.callback_choose_exec, 'choose executable')
        hbox = gtk.HBox(False,5)
        hbox.pack_start(self.entry_exec, True, True, 15)
        hbox.pack_start(button, True, True, 15)
        frame.add(hbox)
        vbox0.pack_start(frame, True, True, 5)

        frame = gtk.Frame('Specify command line options for executable:')
        self.entry_exec_args = gtk.Entry(max=0)
        self.entry_exec_args.set_text(' '.join(parent.executable_args))
        hbox = gtk.HBox(False,5)
        hbox.pack_start(self.entry_exec_args, True, True, 15)
        frame.add(hbox)
        vbox0.pack_start(frame, True, True, 5)

        frame = gtk.Frame('Specify libayudame.so:')
        self.entry_lib = gtk.Entry(max=0)
        if parent.library == None or len(parent.library) == 0:
            self.entry_lib.set_text('#enter libayudame.so here#')
        else:
            self.entry_lib.set_text(parent.library)
        self.entry_lib.set_width_chars(64)
        button = gtk.Button(stock=gtk.STOCK_OPEN)
        button.connect('clicked', self.callback_choose_lib, 'choose library')
        hbox = gtk.HBox(False,5)
        hbox.pack_start(self.entry_lib, True, True, 15)
        hbox.pack_start(button, True, True, 15)
        frame.add(hbox)
        vbox0.pack_start(frame, True, True, 5)

        frame = gtk.Frame('Specify SMPSs configuration file:')
        self.entry_cfg = gtk.Entry(max=0)
        self.entry_cfg.set_text('#enter configuration file here#')
        self.entry_cfg.set_width_chars(64)
        button = gtk.Button(stock=gtk.STOCK_OPEN)
        button.connect('clicked', self.callback_choose_cfg, 'choose configuration file')
        hbox = gtk.HBox(False,5)
        hbox.pack_start(self.entry_cfg, True, True, 15)
        hbox.pack_start(button, True, True, 15)
        frame.add(hbox)
        vbox0.pack_start(frame, True, True, 5)


        frame = gtk.Frame('Specify port:')
        self.entry_port0 = gtk.Entry(max=0)
        try:
            port=os.environ['AYU_PORT']
        except:
            port=int(time.time()%100)+7700
            if (VERBOSE > 0):
                print __progname__, ': AYU_PORT is not defined, setting AYUPORT.'
                print __progname__, ': AYU_PORT set to ', port

        self.entry_port0.set_text(str(port))
        frame.add(self.entry_port0)

        hbox = gtk.HBox(False,5)
        hbox.pack_start(frame, True, True, 0)

        frame = gtk.Frame('Specify number of CPUs:')
        self.entry_cpus = gtk.Entry(max=0)
        try:
            cpus=os.environ['CSS_NUM_CPUS']
        except:
            cpus=2
            if (VERBOSE > 0):
                print __progname__, ': CSS_NUM_CPUS is not defined, setting CSS_NUM_CPUS.'
                print __progname__, ': CSS_NUM_CPUS set to ', cpus
        self.entry_cpus.set_text(str(cpus))
        frame.add(self.entry_cpus)

        hbox.pack_start(frame, True, True, 0)
        vbox0.pack_start(hbox, True, True, 5)

        # notebook tab for remote connection
        vbox1=gtk.VBox()
        hbox = gtk.HBox(False,5)

        frame = gtk.Frame('Specify host:')
        self.entry_host = gtk.Entry(max=0)
        self.entry_host.set_text('localhost')
        frame.add(self.entry_host)
        hbox.pack_start(frame, True, True, 0)

        frame = gtk.Frame('Specify port:')
        self.entry_port1 = gtk.Entry(max=0)
        self.entry_port1.set_text(str(port))
        frame.add(self.entry_port1)
        hbox.pack_start(frame, True, True, 0)

        vbox1.pack_start(hbox, False, True, 5)


        # fill notebook
        self.notebook=gtk.Notebook()
        self.notebook.append_page(vbox0,gtk.Label('local connection'))
        self.notebook.append_page(vbox1,gtk.Label('remote connection'))

        # # MPI frame
        # frame_MPI=gtk.Frame('MPI settings')
        # self.check_MPI=gtk.CheckButton('execute with mpiexec')
        # adjust = gtk.Adjustment(1,1,99,1)
        # self.spin_MPI=gtk.SpinButton(adjust,1,2)
        # hbox=gtk.HBox()
        # hbox.pack_start(self.check_MPI,True,True)
        # hbox.pack_start(self.spin_MPI,False,False)
        # frame_MPI.add(hbox)

        # global layout
        self.vbox.pack_start(self.notebook, True, True, 5)
        # self.vbox.pack_start(frame_MPI, True, True, 5)

        # global adjustments
        self.set_border_width(10)
        self.show_all()

    def callback_choose_exec(self, widget, callback_data):
        choose_dlg = gtk.FileChooserDialog(title='Specify the executable',
                                           action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                           buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,
                                                    gtk.STOCK_OPEN,gtk.RESPONSE_OK))
        response = choose_dlg.run()
        if response == gtk.RESPONSE_OK:
            self.entry_exec.set_text(choose_dlg.get_filename())
        choose_dlg.destroy()

    def callback_choose_lib(self, widget, callback_data):
        choose_dlg = gtk.FileChooserDialog(title='Specify libayudame.so',
                                           action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                           buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,
                                                    gtk.STOCK_OPEN,gtk.RESPONSE_OK))
        response = choose_dlg.run()
        if response == gtk.RESPONSE_OK:
            self.entry_lib.set_text(choose_dlg.get_filename())
        choose_dlg.destroy()

    def callback_choose_cfg(self, widget, callback_data):
        choose_dlg = gtk.FileChooserDialog(title='Specify the configuration file',
                                           action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                           buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,
                                                    gtk.STOCK_OPEN,gtk.RESPONSE_OK))
        response = choose_dlg.run()
        if response == gtk.RESPONSE_OK:
            self.entry_cfg.set_text(choose_dlg.get_filename())
        choose_dlg.destroy()

    def get_exec(self):
        response = self.entry_exec.get_text()
        if response == '' or response[0] == '#':
           return ''
        else:
            return response

    def get_exec_args(self):
        return self.entry_exec_args.get_text()

    def get_lib(self):
        response = self.entry_lib.get_text()
        if response == '' or response[0] == '#':
           return None
        else:
            return response

    def get_cfg(self):
        response = self.entry_cfg.get_text()
        if response == '' or response[0] == '#':
           return None
        else:
            return response

    def get_port(self):
        if self.notebook.get_current_page() == 0: # local
            response = self.entry_port0.get_text()
        else: # remote
            response = self.entry_port1.get_text()
        if len(response) == 0:
           return 7795
        else:
            return int(response)

    def get_host(self):
        response = self.entry_host.get_text()
        if len(response) == 0 or response[0] == '#':
           return 'localhost'
        else:
            return response

    def get_cpus(self):
        response = self.entry_cpus.get_text()
        if len(response) == 0:
           return 2
        else:
            return int(response)

    def get_local_remote(self):
        return self.notebook.get_current_page() # 0:local, 1:remote

#******

#****c* TEMANEJO/ExportDialog
# NAME
#   ExportDialog
#   - dialog for choosing the export file format
# DESCRIPTION
#   With this Dialog one can choose the export file format name
# DERIVED FROM
#   gtk.Dialog
# METHODS
#   __init__
#   callback_choose_file
#   callback_radio_format
#   get_filename
#   get_format
# SOURCE

class ExportDialog(gtk.Dialog):
    def __init__(self,parent):
        gtk.Dialog.__init__(self,title='Export', parent=self,
                            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OK,gtk.RESPONSE_OK))

        self.jpg_quality   = 90
        self.framerate     = 4

        # image export
        frame_img=gtk.Frame('Image export format')
        frame_img.set_border_width(10)
        vbox = gtk.VBox()

        self.radio_png = gtk.RadioButton(None, 'png', use_underline=False)
        self.radio_jpeg = gtk.RadioButton(self.radio_png, 'jpeg', use_underline=False)
        self.radio_dot = gtk.RadioButton(self.radio_png, 'dot', use_underline=False)
        self.radio_png.connect('clicked', self.callback_radio_format, 'png')
        self.radio_jpeg.connect('clicked', self.callback_radio_format, 'jpeg')
        
        vbox.pack_start(self.radio_png)
        vbox.pack_start(self.radio_jpeg)
        frame_img.add(vbox)

        # text export
        frame_text=gtk.Frame('Text export format')
        frame_text.set_border_width(10)
        vbox = gtk.VBox()

        self.radio_nodes_edges = gtk.RadioButton(self.radio_png, '.nodes & .edges', use_underline=False)
        self.radio_dot = gtk.RadioButton(self.radio_png, 'dot', use_underline=False)
        self.radio_nodes_edges.connect('clicked', self.callback_radio_format, 'text')
        self.radio_dot.connect('clicked', self.callback_radio_format, 'dot')

        vbox.pack_start(self.radio_nodes_edges)
        vbox.pack_start(self.radio_dot)
        frame_text.add(vbox)

        # video export
        frame_video=gtk.Frame('Multiple image/video export format')
        frame_video.set_border_width(10)
        vbox = gtk.VBox()

        self.radio_png_ser = gtk.RadioButton(self.radio_png, 'png series', use_underline=False)
        self.radio_avi = gtk.RadioButton(self.radio_png, 'avi', use_underline=False)
        self.radio_gif = gtk.RadioButton(self.radio_png, 'gif', use_underline=False)
        self.radio_png_ser.connect('clicked', self.callback_radio_format, 'png_ser')
        self.radio_avi.connect('clicked', self.callback_radio_format, 'avi')
        self.radio_gif.connect('clicked', self.callback_radio_format, 'gif')

        vbox.pack_start(self.radio_png_ser)
        vbox.pack_start(self.radio_avi)
        vbox.pack_start(self.radio_gif)
        frame_video.add(vbox)

        # settings
        frame_settings=gtk.Frame('Settings')
        frame_settings.set_border_width(10)
        hbox = gtk.HBox()
        label1 = gtk.Label('jpeg quality:')
        adjust = gtk.Adjustment(self.jpg_quality,10,100,5)
        self.spin_quality=gtk.SpinButton(adjust,0,0)
        label2 = gtk.Label('animation frames/s:')
        adjust = gtk.Adjustment(self.framerate,1,30,1)
        self.spin_framerate=gtk.SpinButton(adjust,0,0)

        hbox.pack_start(label1,False,False,0)
        hbox.pack_start(self.spin_quality,True,False,0)
        hbox.pack_start(label2,False,False,0)
        hbox.pack_start(self.spin_framerate,True,False,0)
        frame_settings.add(hbox)

        # file name
        frame_file=gtk.Frame('File name')
        frame_file.set_border_width(10)
        hbox = gtk.HBox(False,5)
        self.entry_file=gtk.Entry()
        self.entry_file.set_size_request(500,-1)
        if parent.executable == None:
            base = 'temanejo'
        else:
            base = parent.executable
        self.entry_file.set_text(base+time.strftime(
                '_%Y%m%d-%H%M%S.png', time.localtime()))
        button = gtk.Button(stock=gtk.STOCK_OPEN)
        button.connect('clicked', self.callback_choose_file, 'choose file')

        hbox.pack_start(self.entry_file, True, True, 15)
        hbox.pack_start(button, True, True, 15)
        frame_file.add(hbox)

        # global layout
        hbox = gtk.HBox()
        hbox.pack_start(frame_text)
        hbox.pack_start(frame_img)
        hbox.pack_start(frame_video)
        self.vbox.pack_start(hbox,True,True,0)
        self.vbox.pack_start(frame_settings,True,True,0)
        self.vbox.pack_start(frame_file, True, True, 5)

        self.show_all()

        self.format='png'

    def callback_radio_format(self,widget, callback_data):
        self.format = callback_data
        if callback_data == 'text':
            ext = '[nodes|edges]'
        else:
            ext = self.format
        filename = self.entry_file.get_text()
        if callback_data == 'png_ser':
            filename = filename.rpartition('.')[0] + '_NNNNN.png'
        else:
            if filename.find('NNNNN') == -1:
                filename = filename.rpartition('.')[0] + '.' + ext
            else:
                filename = filename.replace('_NNNNN','')
                filename = filename.rpartition('.')[0] + '.' + ext

        self.entry_file.set_text(filename)

    def callback_choose_file(self, widget, callback_data):
        choose_dlg = gtk.FileChooserDialog(title='Choose a file',
                                           action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                           buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,
                                                    gtk.STOCK_OPEN,gtk.RESPONSE_OK))
        response = choose_dlg.run()
        if response == gtk.RESPONSE_OK:
            self.entry_file.set_text(choose_dlg.get_filename())
        choose_dlg.destroy()

    def get_format(self):
        return self.format

    def get_filename(self):
        return self.entry_file.get_text()

    def get_framerate(self):
        return self.spin_framerate.get_value_as_int()

    def get_quality(self):
        return self.spin_quality.get_value_as_int()

#******


#****c* TEMANEJO/SettingsDialog
# NAME
#   SettingsDialog
#   - dialog for adjusting the setting of TEMANEJO
# DESCRIPTION
#   This Dialog contains switches for adjusting the behaviour of TEMANEJO
# DERIVED FROM
#   gtk.Dialog
# METHODS
#   __init__
#   get_check_autozoom
#   get_check_key
#   get_edge_colours
#   get_edge_labels
#   get_graph_direction
#   get_spin_margin_width
#   get_spin_edge_width
#   get_spin_node_size
#   get_spin_nsteps
# SOURCE

class SettingsDialog(gtk.Dialog):
    def __init__(self,parent):
        gtk.Dialog.__init__(self,title='Settings', parent=self,
                            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OK,gtk.RESPONSE_OK))

        vbox_ui_left = gtk.VBox()
        vbox_ui_right = gtk.VBox()
        vbox_ne = gtk.VBox()
        vbox_gen = gtk.VBox()
        vbox_tweaks = gtk.VBox()


        # General interface
        frame_general = gtk.Frame('Interface')
        #frame_general.set_tooltip_text('General interface settings')
        frame_general.add(vbox_gen)
        vbox_ui_right.pack_start(frame_general)

        # how many steps to run
        label = gtk.Label('Run n steps')
        label.set_alignment(0.0,0.5)
        label.set_size_request(150,-1)
        adjust = gtk.Adjustment(parent.n_steps,1,1000,1)
        self.spin_nsteps=gtk.SpinButton(adjust,1,0)
        hbox=gtk.HBox()
        hbox.pack_start(label,True, True)
        hbox.pack_start(self.spin_nsteps,False,False)
        #hbox.set_tooltip_text('How many events to advance when running several steps')
        align=gtk.Alignment()
        align.add(hbox)
        align.set_padding(8,8,8,8)
        vbox_gen.pack_start(align)

        # Whether to zoom automatically
        self.check_autozoom=gtk.CheckButton('Autozoom',False)
        self.check_autozoom.set_active(parent.widget.autozoom)
        #self.check_autozoom.set_tooltip_text('Whether to zoom automatically when the graph changes.\n'
        #                                     'Note that the zoom buttons will have no effect when this is checked.')
        align=gtk.Alignment()
        align.add(self.check_autozoom)
        align.set_padding(8,8,8,8)
        vbox_gen.pack_start(align)

        # show markers
        self.check_markers=gtk.CheckButton('Show markers',True)
        self.check_markers.set_active(parent.widget.graphview.show_markers)
        #self.check_markers.set_tooltip_text('Whether to show blocked and priority markers.')
        align=gtk.Alignment()
        align.add(self.check_markers)
        align.set_padding(8,8,8,8)
        vbox_gen.pack_start(align)

        # nodes and edges
        frame_nodes_edges = gtk.Frame('Node appearance')
        #frame_nodes_edges.set_tooltip_text('How to display nodes and their margins\n')

        # Size of nodes
        label = gtk.Label('Size of nodes')
        label.set_alignment(0.0,0.5)
        label.set_size_request(150,-1)
        adjust = gtk.Adjustment(parent.widget.graphview.node_size,1,99,1)
        self.spin_node_size=gtk.SpinButton(adjust,1,0)
        hbox=gtk.HBox()
        hbox.pack_start(label,True, True)
        hbox.pack_start(self.spin_node_size,False,False)
        align=gtk.Alignment()
        align.add(hbox)
        align.set_padding(8,8,8,8)
        vbox_ne.pack_start(align)

        # Thickness of coloured margins
        label = gtk.Label('Margin thickness')
        label.set_alignment(0.0,0.5)
        label.set_size_request(150,-1)
        adjust = gtk.Adjustment(parent.widget.graphview.margin_width,1,99,1)
        self.spin_margin_width=gtk.SpinButton(adjust,1,0)
        hbox=gtk.HBox()
        hbox.pack_start(label,True,True)
        hbox.pack_start(self.spin_margin_width,False,False)
        align=gtk.Alignment()
        align.add(hbox)
        align.set_padding(8,8,8,8)
        vbox_ne.pack_start(align)

        # Thickness of edges
        label = gtk.Label('Edge thickness')
        label.set_alignment(0.0,0.5)
        label.set_size_request(150,-1)
        adjust = gtk.Adjustment(parent.widget.graphview.edge_width,.1,10,.1)
        self.spin_edge_width=gtk.SpinButton(adjust,.1,1)
        hbox=gtk.HBox()
        hbox.pack_start(label,True,True)
        hbox.pack_start(self.spin_edge_width,False,False)
        align=gtk.Alignment()
        align.add(hbox)
        align.set_padding(8,8,8,8)
        vbox_ne.pack_start(align)

        frame_nodes_edges.add(vbox_ne)
        vbox_ui_left.pack_start(frame_nodes_edges)

        # edge colours
        frame_edgecol=gtk.Frame('Edge colours')
        #frame_edgecol.set_tooltip_text('What the edge colours indicate')
        self.radio_ecorigaddr=gtk.RadioButton(None, 'Original dependency address', False)
        self.radio_ecaddr=gtk.RadioButton(self.radio_ecorigaddr, 'Dependency address after renaming', False)
        self.radio_ecnoth=gtk.RadioButton(self.radio_ecorigaddr, 'Nothing', False)
        vbox = gtk.VBox()
        vbox.pack_start(self.radio_ecorigaddr)
        vbox.pack_start(self.radio_ecaddr)
        vbox.pack_start(self.radio_ecnoth)
        ec = parent.widget.graphview.addr_key

        if ec == 'addr':
            self.radio_ecaddr.set_active(True)
        if ec == 'orig_addr':
            self.radio_ecorigaddr.set_active(True)
        if ec == 'nothing':
            self.radio_ecnoth.set_active(True)

        frame_edgecol.add(vbox)
        vbox_ui_left.pack_start(frame_edgecol)

        # edge labels
        frame_edgelab=gtk.Frame('Edge labels')
        self.check_edgelab=gtk.CheckButton('draw edge labels',False)
        self.check_edgelab.set_active(parent.widget.graphview.draw_edge_labels)
        frame_edgelab.add(self.check_edgelab)
        vbox_ui_left.pack_start(frame_edgelab)
        
        # graph direction
        frame_graph_dir=gtk.Frame('Graph direction')
        #frame_graph_dir.set_tooltip_text('How to arrange the graph')

        self.radio_t2b=gtk.RadioButton(None, 'Top to bottom', False)
        self.radio_l2r=gtk.RadioButton(self.radio_t2b, 'Left to right', False)
        self.radio_b2t=gtk.RadioButton(self.radio_t2b, 'Bottom to top', False)
        self.radio_r2l=gtk.RadioButton(self.radio_t2b, 'Right to left', False)
        vbox = gtk.VBox()
        vbox.pack_start(self.radio_t2b)
        vbox.pack_start(self.radio_l2r)
        vbox.pack_start(self.radio_b2t)
        vbox.pack_start(self.radio_r2l)
        direction=parent.widget.graphview.layout_direction
        if direction == 't2b':
            self.radio_t2b.set_active(True)
        elif direction == 'l2r':
            self.radio_l2r.set_active(True)
        elif direction == 'b2t':
            self.radio_b2t.set_active(True)
        elif direction == 'r2l':
            self.radio_r2l.set_active(True)
        frame_graph_dir.add(vbox)
        vbox_ui_right.pack_start(frame_graph_dir)

        # global layout of user interface tab
        hbox=gtk.HBox()
        hbox.pack_start(vbox_ui_left,padding=5)
        hbox.pack_start(vbox_ui_right,padding=5)

        # tweaks tab

        # layout routine
        frame_layout = gtk.Frame('Graph layout')

        self.radio_temanejo=gtk.RadioButton(None, 'Use Temanejo\'s routine', False)
        self.radio_pygraphviz=gtk.RadioButton(self.radio_temanejo, 'Use pygraphviz (NetworkX)', False)
        self.radio_linear=gtk.RadioButton(self.radio_temanejo, 'Use extended linear visualisation (beta)', False)
        vbox = gtk.VBox()
        vbox.pack_start(self.radio_temanejo)
        vbox.pack_start(self.radio_pygraphviz)
        vbox.pack_start(self.radio_linear)

        if parent.widget.graphview.use_layout == 'temanejo':
            self.radio_temanejo.set_active(True)
        elif parent.widget.graphview.use_layout == 'pygraphviz':
            self.radio_pygraphviz.set_active(True)
        elif parent.widget.graphview.use_layout == 'linear':
            self.radio_linear.set_active(True)
        frame_layout.add(vbox)

        vbox_tweaks.pack_start(frame_layout,expand=False, fill=False)

        self.radio_temanejo.connect('clicked',self.on_graph_layout, 'temanejo')
        self.radio_pygraphviz.connect('clicked',self.on_graph_layout, 'pygraphviz')
        self.radio_linear.connect('clicked',self.on_graph_layout, 'linear')


        # global tasks
        frame_globaltasks = gtk.Frame('Global tasks')

        self.radio_globaltasks_everywhere=gtk.RadioButton(None, 'Put global tasks close to their dependencies', False)
        self.radio_globaltasks_firstrow=gtk.RadioButton(self.radio_globaltasks_everywhere, 'Put global tasks in the first row', False)
        vbox = gtk.VBox()
        vbox.pack_start(self.radio_globaltasks_everywhere)
        vbox.pack_start(self.radio_globaltasks_firstrow)

        if parent.widget.graphview.global_tasks_in_first_row == False:
            self.radio_globaltasks_everywhere.set_active(True)
        elif parent.widget.graphview.global_tasks_in_first_row == True:
            self.radio_globaltasks_firstrow.set_active(True)
        frame_globaltasks.add(vbox)

        if self.radio_pygraphviz.get_active() == True:
            self.radio_globaltasks_everywhere.set_sensitive(False)
            self.radio_globaltasks_firstrow.set_sensitive(False)

        vbox_tweaks.pack_start(frame_globaltasks,expand=False, fill=False)


        # build notebook

        notebook=gtk.Notebook()
        notebook.append_page(hbox,gtk.Label('User Interface'))
        notebook.append_page(vbox_tweaks,gtk.Label('Tweaks'))

        #~ align=gtk.Alignment()
        #~ align.add(hbox)
        #~ align.set_padding(8,8,8,8)
        #~ frame.add(align)
        self.vbox.pack_start(notebook,True,True,0)
        self.show_all()

    def on_graph_layout(self,widget, data=None):
        if data == 'temanejo':
            self.radio_globaltasks_everywhere.set_sensitive(True)
            self.radio_globaltasks_firstrow.set_sensitive(True)
        else:
            self.radio_globaltasks_everywhere.set_sensitive(False)
            self.radio_globaltasks_firstrow.set_sensitive(False)

    def get_check_key(self):
        return self.check_key.get_active()

    def get_check_autozoom(self):
        return self.check_autozoom.get_active()

    def get_check_markers(self):
        return self.check_markers.get_active()

    def get_spin_margin_width(self):
        return self.spin_margin_width.get_value_as_int()

    def get_spin_edge_width(self):
        return self.spin_edge_width.get_value()

    def get_spin_node_size(self):
        return self.spin_node_size.get_value_as_int()

    def get_spin_nsteps(self):
        return self.spin_nsteps.get_value_as_int()

    def get_graph_direction(self):
        if self.radio_t2b.get_active():
            return 't2b'
        if self.radio_l2r.get_active():
            return 'l2r'
        if self.radio_b2t.get_active():
            return 'b2t'
        if self.radio_r2l.get_active():
            return 'r2l'

    def get_edge_colours(self):
        if self.radio_ecaddr.get_active():
            return 'addr'
        if self.radio_ecorigaddr.get_active():
            return 'orig_addr'
        if self.radio_ecnoth.get_active():
            return 'nothing'

    def get_edge_labels(self):
        return self.check_edgelab.get_active()

    def get_graph_layout(self):
        if self.radio_temanejo.get_active():
            return 'temanejo'
        elif self.radio_pygraphviz.get_active():
            return 'pygraphviz'
        elif self.radio_linear.get_active():
            return 'linear'


    def get_global_tasks_first_row(self):
        return self.radio_globaltasks_firstrow.get_active()

#******

#****c* TEMANEJO/AboutDialog
# NAME
#   AboutDialog
#   - dialog displaying information about TEMANEJO
# DESCRIPTION
#   This Dialog displays license information about TEMANEJO
# DERIVED FROM
#   gtk.Dialog
# METHODS
#   __init__
# SOURCE

class AboutDialog(gtk.Dialog):
    def __init__(self):
        gtk.Dialog.__init__(self,title='About TEMANEJO...', parent=self,
                            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            buttons= (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        label=gtk.Label()
        frame=gtk.Frame()
        label.set_text('TEMANEJO version '+ __version__+ '\n'
                       '-------------------------------------------\n\n'
                       '(c) HLRS, University of Stuttgart\n'
                       'published underr the terms of the BSD license.\n\n'
                       'authors: Steffen Brinkmann <brinkmann@hlrs.de>\n\n'
                       'Please refer to the LICENSE file for detailed information.')
        align=gtk.Alignment()
        frame.set_border_width(10)

        image=gtk.Image()
        image.set_from_file(os.path.join(temanejo_base_dir,'resources/temanejo_icon.png'))

        hbox=gtk.HBox()
        hbox.pack_start(image,padding=5)
        hbox.pack_start(label,padding=5)
        align.add(hbox)
        align.set_padding(20,20,10,10)
        frame.add(align)
        self.vbox.pack_start(frame,True,True,0)
        self.show_all()
#******

#****c* TEMANEJO/HelpDialog
# NAME
#   HelpDialog
#   - dialog displaying a short help screen about TEMANEJO
# DESCRIPTION
#   This Dialog displays some helping information about TEMANEJO.
#   For an extensive documentation refer to the files in Tools/doc/
# DERIVED FROM
#   gtk.Dialog
# METHODS
#   __init__
# SOURCE

# class HelpDialog(gtk.Dialog):
#     def __init__(self):
#         gtk.Dialog.__init__(self,title='Help', parent=self,
#                             flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
#                             buttons= (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
#         table=gtk.Table(4,2)
#         image=gtk.Image()
#         image.set_from_file(os.path.join(temanejo_base_dir,'task_NQ.png'))
#         label=gtk.Label('task which is not queued')
#         label.set_alignment(0.0,0.5)
#         label.set_width_chars(25)
#         table.attach(image,0,1,0,1,xpadding=5,ypadding=3)
#         table.attach(label,1,2,0,1)

#         image=gtk.Image()
#         image.set_from_file(os.path.join(temanejo_base_dir,'task_Q.png'))
#         label=gtk.Label('task which is queued')
#         label.set_alignment(0.0,0.5)
#         label.set_width_chars(25)
#         table.attach(image,0,1,1,2,xpadding=5,ypadding=3)
#         table.attach(label,1,2,1,2)

#         image=gtk.Image()
#         image.set_from_file(os.path.join(temanejo_base_dir,'task_R.png'))
#         label=gtk.Label('task which is running')
#         label.set_alignment(0.0,0.5)
#         label.set_width_chars(25)
#         table.attach(image,0,1,2,3,xpadding=5,ypadding=3)
#         table.attach(label,1,2,2,3)

#         image=gtk.Image()
#         image.set_from_file(os.path.join(temanejo_base_dir,'task_F.png'))
#         label=gtk.Label('task which has finished')
#         label.set_alignment(0.0,0.5)
#         label.set_width_chars(25)
#         table.attach(image,0,1,3,4,xpadding=5,ypadding=3)
#         table.attach(label,1,2,3,4)

#         align=gtk.Alignment()
#         align.set_padding(5,5,5,5)
#         align.add(table)

#         frame=gtk.Frame('Graph key')
#         frame.set_border_width(10)
#         frame.add(align)

#         self.vbox.pack_start(frame,True,True,0)
#         self.show_all()
#******


#****c* TEMANEJO/Communicator
# NAME
#   Communicator
#   - thread class for receiving messages from AYUDAME
# DESCRIPTION
#   This class opens a thread for receiving mesages from AYUDAME. It implements
#   a socket cleint. Furthermore it precesses the received messages.
# DERIVED FROM
#   Thread
# METHODS
#   __init__
#   print_and_log
#   run
#   send_request
#   terminate
# SOURCE

class Communicator(Thread):
    def __init__ (self,sock,par):
        Thread.__init__(self)
        self.stopevent = Event()
        self.working=False
        self.sizeofdata = 8 # sizeof(uint64_t)
        self.block_size = 8 * self.sizeofdata
        self.buf_size = self.block_size #* 100 # BUG: != 1 makes it unstable
        self.func_name_buf_size = 128
        self.s=sock
        self.parent=par
        self.event_count=0
        self.log_dt = time.time()

        # send breakpoint requests
        if self.parent.check_anyEvent.get_active():
            self.send_request(0,'AYU_PAUSEONEVENT',[event_t.index('AYU_EVENT_NULL'),1])
        else:
            self.send_request(0,'AYU_PAUSEONEVENT',[event_t.index('AYU_EVENT_NULL'),0])

        if self.parent.check_preruntask.get_active():
            self.send_request(0,'AYU_PAUSEONEVENT',[event_t.index('AYU_PRERUNTASK'),1])
        else:
            self.send_request(0,'AYU_PAUSEONEVENT',[event_t.index('AYU_PRERUNTASK'),0])

        if self.parent.check_removetask.get_active():
            self.send_request(0,'AYU_PAUSEONEVENT',[event_t.index('AYU_REMOVETASK'),1])
        else:
            self.send_request(0,'AYU_PAUSEONEVENT',[event_t.index('AYU_REMOVETASK'),0])

        if self.parent.check_waiton.get_active():
            self.send_request(0,'AYU_PAUSEONEVENT',[event_t.index('AYU_WAITON'),1])
        else:
            self.send_request(0,'AYU_PAUSEONEVENT',[event_t.index('AYU_WAITON'),0])

        if self.parent.check_barrier.get_active():
            self.send_request(0,'AYU_PAUSEONEVENT',[event_t.index('AYU_BARRIER'),1])
        else:
            self.send_request(0,'AYU_PAUSEONEVENT',[event_t.index('AYU_BARRIER'),0])

    def run(self):
        parent = self.parent
        graph = parent.graph
        while not self.stopevent.isSet():
            # receive messages
            self.working=False
            n=''
            n+=self.s.recv(self.buf_size)
            self.working=True
            # -------------------

            # process messages
            buf_count=0
            while len(n) >= self.block_size and \
                     buf_count + self.block_size <= len(n):

                buf=struct.unpack('>qqqqqqqq',n[buf_count:buf_count+self.block_size]) # '>' converts from big-endian to local endianity
                if len(buf) == 0:
                    if VERBOSE > -1:
                        print __progname__, ': struct.unpack failed. Exiting.'
                    return
                buf_count+=self.block_size

#                print buf
                runtimeid = buf[0] # runtime Id
                taskid = buf[1] # task Id
                event = buf[2] # event Id
                ts = buf[7] # timestamp
                parent.last_event=event_t[event]
                parent.last_taskId=taskid

                if event == event_t.index('AYU_ADDTASK') :
                    #print "ADD_TASK",buf[2],buf[3],buf[4]
                    funcId = buf[3]
                    crit = buf[4]
                    # buf[5] is length of source file name
                    # buf[6] is line number
                    srcfilename_buf=''
                    if buf[5] != 0:
                        # read source file name
                        n+=self.s.recv(buf[5])
                        fmt_string=str(buf[5])+'s'
                        srcfilename_buf=struct.unpack(fmt_string,n[buf_count:buf_count+buf[5]])[0]
                        # advance buffer counter
                        buf_count+=buf[5]
                        
                    graph.add_task(taskid,funcId,crit,parent.barrier_count, ts,srcfilename_buf,buf[6])
                    if funcId not in graph._funcName.keys():
                        parent.register_function(funcId,'function %d'%funcId)
                    parent.func_counter[funcId] += 1
                    parent.func_counter_NQ[funcId] += 1
#                    parent.thread_counter_NQ += 1
                    parent.status_counter_NQ += 1
                    self.print_and_log(event_t[event], taskid, funcId)

                elif event == event_t.index('AYU_ADDDEPENDENCY') :
                    graph.add_dep(taskid,buf[3],buf[4],buf[5])
                    self.print_and_log(event_t[event], taskid, buf[3])

                elif event == event_t.index('AYU_ADDTASKTOQUEUE') :
                    if graph.get_status(taskid) == 0:
                        graph.set_threadId(taskid,buf[3])
                        graph.set_status(taskid,1,ts) # queued
                        if graph.get_funcId(taskid) != -1 :
                            parent.func_counter_NQ[graph.get_funcId(taskid)] -= 1
    #                        parent.thread_counter_NQ -= 1
                            parent.status_counter_NQ -= 1
                            parent.func_counter_Q[graph.get_funcId(taskid)]  += 1
                            parent.thread_counter_Q[buf[3]] += 1
                            parent.status_counter_Q  += 1
                        self.print_and_log(event_t[event], taskid, buf[3])

                elif event == event_t.index('AYU_PRESELECTTASK') :
                    pass
#                    self.print_and_log(event_t[event], taskid)

                elif event == event_t.index('AYU_PRERUNTASK') :
                    if graph.get_funcId(taskid) != -1 :
                        parent.func_counter_Q[graph.get_funcId(taskid)] -= 1
                        parent.thread_counter_Q[graph.get_threadId(taskid)]  -= 1
                        if graph.get_status(taskid) == 0: # NQ
                            parent.status_counter_NQ -= 1
                        elif graph.get_status(taskid) == 1: # Q
                            parent.status_counter_Q -= 1
                        parent.func_counter_R[graph.get_funcId(taskid)] += 1
                        parent.thread_counter_R[buf[3]] += 1
                        parent.thread_counter[buf[3]] += 1
                        parent.status_counter_R += 1
                    graph.set_status(taskid,2,ts) # running
                    graph.set_threadId(taskid,buf[3])
                    parent.last_check_event=event_t[event]
                    self.print_and_log(event_t[event], taskid, buf[3])

                elif event == event_t.index('AYU_RUNTASK') :
                    graph.set_status(taskid,2,ts) # running TODO

                elif event == event_t.index('AYU_POSTRUNTASK') :
                    self.print_and_log(event_t[event], taskid)

                elif event == event_t.index('AYU_REMOVETASK') :
                    graph.set_status(taskid,3,ts) # finished
                    if graph.get_funcId(taskid) != -1 :
                        parent.func_counter_R[graph.get_funcId(taskid)] -= 1
                        parent.thread_counter_R[graph.get_threadId(taskid)]  -= 1
                        parent.status_counter_R -= 1
                        parent.func_counter_F[graph.get_funcId(taskid)] += 1
                        parent.thread_counter_F[graph.get_threadId(taskid)] += 1
                        parent.status_counter_F += 1
                        parent.last_check_event=event_t[event]
                        if taskid in graph.dependentTasks:
                            for t in graph.dependentTasks[taskid]:
                                parent.comm.send_request(0,'AYU_BLOCKTASK',[t,0])
                                if t in graph.blockedTasks:
                                    graph.blockedTasks.remove(t)

                    self.print_and_log(event_t[event], taskid)
                    

                elif event == event_t.index('AYU_RUNTASKFAILED') :
                #     parent.func_counter_R[graph.funcId[taskid]] -=1
                #     parent.thread_counter_R[graph.threadId[taskid]] -= 1
                #     parent.status_counter_R -= 1
                #     parent.func_counter_X[graph.funcId[taskid]] +=1
                #     parent.thread_counter_X[graph.threadId[taskid]] += 1
                #     parent.status_counter_X += 1
                #     graph.set_status(taskid,4,ts) # failed
                    self.print_and_log(event_t[event], taskid)

                elif event == event_t.index('AYU_WAITON') :
                    graph.add_task(taskid,-1,0,parent.barrier_count,ts)
                    parent.last_check_event=event_t[event]
                    self.print_and_log(event_t[event], taskid)

                elif event == event_t.index('AYU_BARRIER') :
                    parent.barrier_count += 1
                    graph.total_barrier_count += 1
                    parent.last_check_event=event_t[event]
                    self.print_and_log(event_t[event], taskid)

                elif event == event_t.index('AYU_INIT') :
                    self.print_and_log(event_t[event], taskid)

                elif event == event_t.index('AYU_PREINIT') :
                    self.print_and_log(event_t[event], taskid)

                elif event == event_t.index('AYU_REGISTERTASK') :
                    # read function name
                    n+=self.s.recv(buf[3])
                    fmt_string=str(buf[3])+'s'
                    funcname_buf=struct.unpack(fmt_string,n[buf_count:buf_count+buf[3]])[0]
                    # advance buffer counter
                    buf_count+=buf[3]

                    # register function
                    parent.register_function(buf[4],funcname_buf)

                    # print and log
                    self.print_and_log(event_t[event], taskid, buf[4], funcname_buf)

                elif event == event_t.index('AYU_ADDHIDDENTASK') :
                    graph.add_task(taskid,-1,0,parent.barrier_count,ts)
#                    parent.func_counter[buf[3]] += 1
#                    parent.func_counter_NQ[buf[3]] += 1
                    self.print_and_log(event_t[event], taskid)

                elif event == event_t.index('AYU_ADDWAITONTASK') :
                    graph.add_task(taskid,-1,0,parent.barrier_count,ts)
                    self.print_and_log(event_t[event], taskid)

                elif event == event_t.index('AYU_FINISH') :
                    self.working=False
                    self.print_and_log(event_t[event], taskid)
                    time.sleep(.1)
                    parent.update_ui('all')

                    try:
                        #self.s.shutdown(socket.SHUT_RDWR);
                        self.s.close();
                    except socket.error, msg:
                        if (VERBOSE > -1):
                            print __progname__, ': Error closing connection: ', msg
#                    parent.widget.queue_draw()
                    return

                else:
                    if (VERBOSE > 0):
                        print  __progname__, ': ERRROR: unknown event (',event,') received for taskId: ', taskid
                    return
            # -----------------------------------------------


    def print_and_log(self,event,taskid,a=None,b=None):

        if event == 'AYU_ADDDEPENDENCY':
            t='\n%s,\t taskId %i,\t depId: %i' % (event, taskid, a)
#            if VERBOSE > 2:
#                print __progname__, ':', event ,'received for taskId:', taskid, ', depId:', a

        elif event == 'AYU_PRERUNTASK' or \
             event == 'AYU_ADDTASKTOQUEUE':
            t='\n%s,\t taskId %i,\t threadId: %i' % (event, taskid, a)
#            if VERBOSE > 2:
#                print __progname__, ':', event ,'received for taskId:', taskid, ', threadId:', a

        elif event == 'AYU_ADDTASK':
            t='\n%s,\t taskId %i,\t funcId: %i' % (event, taskid, a)
#            if VERBOSE > 2:
#                print __progname__, ':', event ,'received for taskId:', taskid, ', funcId:', a

        elif event == 'AYU_REMOVETASK':
            t='\n%s,\t taskId %i' % (event,taskid)
#            if VERBOSE > 1:
#                print __progname__, ':', event ,'received for taskId:', taskid
            #DEBUG:
            #~status = self.parent.graph._status.values()
#            print status
            #~print "DEBUG - TEMANEJO: ",status.count(0),\
                #~status.count(1),\
                #~status.count(2),\
                #~status.count(3),\
                #~status.count(4)

        elif event == 'AYU_REGISTERTASK':
            t='\n%s,\t taskId %i,\t funcId %i, function name: %s' % (event, taskid, a, b)
#            if VERBOSE > 1:
#                print __progname__, ':', event ,'received for taskId:', taskid, ', funcId:', a\
#                    , 'function name:', b

        elif event == 'AYU_ADDWAITONTASK':
             t='\n%s,\t taskId %i' % (event, taskid)
#            if VERBOSE > 1:
#                print __progname__, ':', event ,'received for taskId:', taskid

        elif event == 'AYU_FINISH':
            t='\n%s,\t taskId %i\n### Exiting ###' % (event,taskid)
#            if VERBOSE > 1:
#                print __progname__, ':', event ,'received for taskId:', taskid


        else:
            t='\n%s,\t taskId: %i' % (event,taskid)
#            if VERBOSE > 2:
#                print __progname__, ':', event ,'received for taskId:', taskid

        self.parent.prelog += t

#        self.parent.update_ui('statusbar')
#        self.parent.update_ui('log')

    def send_request(self,t_id,req,data=None):
        d=[0,0,0,0,0,0]
        if data != None:
            if isinstance(data,list):
                for i in range(len(data)):
                    d[i]=data[i]
            else:
                d[0]=data

        buf=struct.pack('>qqqqqqqq',t_id,request_t.index(req),
                        d[0],d[1],d[2],d[3],d[4],d[5])
        try:
            self.s.send(buf)
            if (VERBOSE > 1):
                print __progname__, ': sent:', req,data
        except socket.error, msg:
            if (VERBOSE > -1):
                print __progname__, ': Error sending to server:', msg

    def terminate(self):
        self.stopevent.set()

#******

#****f* TEMANEJO/Temanejo_main
# NAME
#   Temanejo_main
#   - the main application class
# DESCRIPTION
#   This class is the main application class. It implements the main function
#   and a parser for command line options. The main function creates a
#   TemanejoWindow and run gtk.main()
# METHODS
#   init_optparser
#   main
# SOURCE

class Temanejo_main:

    def main(self):

        sys.setcheckinterval(1000)

        # initiaise  option parser
        parser = self.init_optparser()

        # intitiate window
        win = TemanejoWindow()
        win.connect('destroy', win.on_quit)
        win.connect('delete_event', win.on_quit)

        # process options
        (options, args) = parser.parse_args(sys.argv[1:])


        win.set_title(__progname__+' '+options.title)
        win.widget.graphview.layout_direction = options.layoutdir
        win.widget.autozoom = options.autozoom
        win.widget.graphview.addr_key = options.edge_col

        win.widget.graphview.node_colours = options.node_col
        win.widget.graphview.margins_mean = options.margin_col
        win.widget.graphview.shapes_mean = options.shapes
        win.widget.graphview.use_pygraphviz = options.pygraphviz
        win.widget.graphview.global_tasks_in_first_row = options.global_tasks

        if len(options.lib_file):
            win.library = options.lib_file

        global VERBOSE
        VERBOSE = options.verbose

        if options.quiet:
            VERBOSE = -1

        # run program
        if VERBOSE > -1:
            print '#',__progname__, ': Start session,', time.ctime()

        if len(args) >= 1:
            win.executable=args[0]
        if len(args) > 1:
            win.executable_args=args[1:]
        for i in range(len(win.executable_args)):
            if win.executable_args[i].find(' ') != -1:
                win.executable_args[i]= '"'+win.executable_args[i]+'"'

        win.on_connect(None)

        gtk.gdk.threads_init()
        gtk.gdk.threads_enter()
        gtk.main()
        gtk.gdk.threads_leave()

    def init_optparser(self):

        parser = optparse.OptionParser(
            usage='%prog [options] [smpss_app [smpss_app_args]]',
            description='TEMANEJO is designed to connect to an SMPSs instrumented application '
            'which has been linked to the AYUDAME library. The SMPSs application can either be '
            'run independently on the same or a remote computer, or can be given to TEMANEJO '
            'as a command line option and started by TEMANEJO. Please refer to the README file '
            'and the documentation in the doc/ directory.',
            epilog='TEMANEJO is published under the terms of the BSD license. '
            '(C) HLRS, University of Stuttgart. For further information please refer to the '
            'LICENSE file.',
            version='%%prog %s' % __version__)
        parser.add_option(
            '-v', '--verbose',
            action='count', dest='verbose', default=0,
            help='Switch on verbose output. Use -vv and -vvv for increasingly epic information')
        parser.add_option(
            '-q', '--quiet',
            action='store_true', dest='quiet', default=False,
            help='Switch off any output except error messages')
        parser.add_option('-t', '--title', dest='title', default='',
                      help='window title (appended to "TEMANEJO:")')
        parser.add_option(
            '-d', '--layout-direction',
            type='choice', choices=('t2b', 'l2r', 'b2t', 'r2l'),
            dest='layoutdir', default='t2b',
            help='how to draw the graph: top to bottom (\'t2b\'), left to right (\'l2r\'),'
            'Bottom to top (\'b2t\') or right to left (\'r2l\') [default: %default]')
        parser.add_option(
            '-Z', '--autozoom',
            action='store_true', dest='autozoom', default=False,
            help='Switch on autozooming of the graph')
        parser.add_option(
            '-z', '--no-autozoom',
            action='store_false', dest='autozoom', default=False,
            help='Switch off autozooming of the graph [default]')
        parser.add_option(
            '-n', '--node-colour',
            type='choice', choices=('nothing','status','thread','function', 'task_duration'),
            dest='node_col', default='function',
            help='Choose what the node colour means. One of "nothing" (no colours drawn), "status" (colour indicates task status), "thread" (thread Id), "function" [default] (function Id), "task_dur" (task duration)')
        parser.add_option(
            '-m', '--margin-colour',
            type='choice', choices=('nothing','status','thread','function', 'task_duration'),
            dest='margin_col', default='status',
            help='Choose what the margin colour means. One of "nothing" (no margins drawn), "status" [default] (margin indicates task status), "thread" (thread Id), "function" (function Id), "task_dur" (task duration)')
        parser.add_option(
            '-e', '--edge-colour',
            type='choice', choices=('nothing','addr','orig_addr'),
            dest='edge_col', default='orig_addr',
            help='Choose what the edge colour means. One of "nothing" (black edges), "orig_addr" [default] (original address of dependency), "addr" (address of dependency after renaming)')
        parser.add_option(
            '-s', '--shapes',
            type='choice', choices=('nothing','thread','function'),
            dest='shapes', default='thread',
            help='Choose what the node shape means. One of "nothing" (no margins drawn), "thread" [default] (thread Id), "function" (function Id)')
        parser.add_option(
            '-L', '--library',
            action='store', dest='lib_file', default='',
            help='Specify library file to communicate with the application. By default, libayudame.so is expected in temanejo_dir/lib/.')
        parser.add_option(
            '', '--pygraphviz',
            action='store_true', dest='pygraphviz', default=False,
            help='Specify whether to use pygraphviz for the graph layout. Pygraphviz is part of the networkx package, but may cause problems on some systems.')
        parser.add_option(
            '', '--global-tasks-in-first-row',
            action='store_true', dest='global_tasks', default=False,
            help='Specify whether global tasks should be drawn in first line.')

        return parser


if __name__ == '__main__':

    temanejo_app = Temanejo_main()
    #cProfile.run('temanejo_app.main()','profile.out','time')
    temanejo_app.main()

#******
