#****h* TEMANEJO/TEMANEJO_GRAPH
# NAME
#   TEMANEJO_GRAPH
#   - the graph class for *Ss' task graph
# DESCRIPTION
#   temanejo_graph.py implements a graph data structure derived from the
#   MultiDiGraph (directed graph with multiple edges) structure from
#   NetworkX.
# PORTABILITY
#   The following modules are imported:
#   * cairo
#   * networkx as nx
#   * colorsys
#   * time
# AUTHOR
#	Steffen Brinkmann, HLRS <brinkmann@hlrs.de>
# COPYRIGHT
#	(C) HLRS, University of Stuttgart
#	temanejo_graph.py is published under the terms of the BSD license.
#******

VERBOSE_GRAPH=False

import networkx as nx

#****c* TEMANEJO_GRAPH/Graph
# NAME
#   Graph
#   - class for the dependency graph of a *Ss application
# DESCRIPTION
#   Graph implements a data structure representing an directed graph.
# DERIVED FROM
#   NetworkX.MultiDiGraph
# METHODS
#   __init__
#   get_size
#   add_task
#   add_dep
#   set_status
#   set_threadId
#   set_funcname
#   draw
#   draw_edges
#   draw_nodes
#   drawBox
#   drawShape
#   drawMargin
#   drawTriangle
#   drawDiamond
#   draw_labels
#   get_task_from_xy
# SOURCE

class Graph(nx.MultiDiGraph):


	def __init__(self):
		nx.MultiDiGraph.__init__(self)

		self.last_nodeId	   = 0	# last nodeId used. New tasks/nodes have to use last_nodeId + 1
		self._taskId		   = {}   # task Id, used as label for nodes
		self._nodeId_of_taskId = {}   # node Id for every task. For reverse lookup of self.taskId
		self._funcId		   = {}   # function Id
		self._threadId		   = {}   # thread Id
		self._status		   = {}   # 0: not queued, 1: queued, 2: running, 3: finished, 4: failed
		self._funcName		   = {}   # list of function names
		self.addresses		   = {}   # list of memory addresses of dependencies
		self.orig_addresses    = {}   # list of original memory addresses of dependencies (before renaming)
		self.blockedTasks	   = []   # tasks to be marked as blocked
		self.dependentTasks    = {}   # tasks that depend on index task due to user input
		self.criticalTasks	   = []   # tasks to be marked as critical (prioritised)
		self._barrier_count	   = {}   # count of barriers before each task
		self.total_barrier_count   = 0    # number of barriers in graph
		self._highlight		   = {}   # highlighted 1: neighbour, 2: clicked
		self.is_invalid	       = True # True when graph has changed, decides whether to generate new layout


#	def set_widget(self,widget):
#		self.widget	   = widget # parent widget

	def clear(self):
		nx.MultiDiGraph.clear(self)

		self.last_nodeId = 0
		self._taskId = {}
		self._nodeId_of_taskId = {}
		self._funcId = {}
		self._threadId = {}
		self._status = {}
		self._funcName = {}
		self.addresses = {}
		self.orig_addresses = {}
		self.blockedTasks = []
		self.criticalTasks = []
		self.dependentTasks = {}
		self._barrier_count = {}
		self._highlight = {}
		self.is_invalid = True # True when graph has changed

	def add_task(self,taskId, funcId, critical, barrier_count, timestamp, filename='', linenumber=0):
		# print "add_task" ,taskId, funcId, critical, barrier_count, timestamp, filename, linenumber
		self.last_nodeId += 1
		nodeId = self.last_nodeId

		self.add_node(nodeId, ts_add=timestamp)
		self._taskId[nodeId]		   = taskId
		self._nodeId_of_taskId[taskId] = nodeId
		self._funcId[nodeId]		   = funcId
		self._threadId[nodeId]		   = -1
		self._status[nodeId]		   = 0 # not queued
		self._highlight[nodeId]		   = 0
		if critical == 1:
			self.criticalTasks.append(nodeId)
		self._barrier_count[nodeId] = barrier_count
		self.is_invalid=True

	def add_dep(self,taskId, lastWriter, address, original_address):
		self.add_edge(self._nodeId_of_taskId[lastWriter],
						self._nodeId_of_taskId[taskId],
						addr=address,
						orig_addr=original_address)
		if address not in self.addresses:
			self.addresses[address]=0
		self.addresses[address]+=1
		if original_address not in self.orig_addresses:
			self.orig_addresses[original_address]=0
		self.orig_addresses[original_address]+=1
		self.is_invalid=True

	def get_taskId(self,nodeId):
		return self._taskId[nodeId]

	def get_nodeId(self,taskId):
		return self._nodeId_of_taskId[taskId]

	def get_status(self,taskId):
		nodeId = self._nodeId_of_taskId[taskId]
		return self._status[nodeId]

	def set_status(self,taskId,status, timestamp):
		nodeId = self._nodeId_of_taskId[taskId]
		if status == 1:
			self.add_node(nodeId, ts_q=timestamp)
		elif status == 2:
			self.add_node(nodeId, ts_r=timestamp)
		elif status == 3:
			self.add_node(nodeId, ts_f=timestamp)
		elif status == 4:
			self.add_node(nodeId, ts_fail=timestamp)
		self._status[nodeId]=status

	def get_threadId(self,taskId):
		nodeId = self._nodeId_of_taskId[taskId]
		return self._threadId[nodeId]

	def set_threadId(self,taskId,threadId):
		nodeId = self._nodeId_of_taskId[taskId]
		self._threadId[nodeId]=threadId

	def get_funcId(self,taskId):
		nodeId = self._nodeId_of_taskId[taskId]
		return self._funcId[nodeId]

	def get_funcname(self,funcId):
		if funcId == -1:
			return "wait on task"
		return self._funcName[funcId]

	def set_funcname(self,funcId,buf):
		self._funcName[funcId] = buf

	def get_highlight(self,taskId):
		nodeId = self._nodeId_of_taskId[taskId]
		return self._highlight[nodeId]

	def get_barrier_count(self,taskId):
		nodeId = self._nodeId_of_taskId[taskId]
		return self._barrier_count[nodeId]

	def distance(self,source,target):
		if target == None or source == None:
			return None
		try:
			return nx.shortest_path_length(self,source,target)
		except nx.NetworkXNoPath:
			try:
				return -nx.shortest_path_length(self,target,source)
			except nx.NetworkXNoPath:
				return None
			except nx.NetworkXError:
				return None
		except nx.NetworkXError:
			return None


#******
